import json

from builtins import bool
from typing import Any, Dict, List, Optional, Type

import arrow
import inspect
from functools import reduce

from canvas_workflow_kit.patient_recordset import (
    AdministrativeDocumentRecordSet,
    AllergyIntoleranceRecordSet,
    PatientRecordSet,
    BillingLineItemRecordSet,
    ConditionRecordSet,
    ImagingReportRecordSet,
    ImmunizationRecordSet,
    InpatientStayRecordSet,
    InstructionRecordSet,
    InterviewRecordSet,
    LabReportRecordSet,
    MedicationRecordSet,
    MessageRecordSet,
    ProcedureRecordSet,
    ProtocolOverrideRecordSet,
    PrescriptionRecordSet,
    ReasonForVisitRecordSet,
    ReferralRecordSet,
    ReferralReportRecordSet,
    UpcomingAppointmentNoteRecordSet,
    UpcomingAppointmentRecordSet,
    VitalSignRecordSet
)
from canvas_workflow_kit.internal.attrdict import to_attr_dict
from canvas_workflow_kit.timeframe import Timeframe
from canvas_workflow_kit.value_set.value_set import ValueSet

# For has_visit_within
from canvas_workflow_kit.value_set.v2022 import encounter

# from protocols.value_set.v2018 import (DischargedToHealthCareFacilityForHospiceCare,
#                                  DischargedToHomeForHospiceCare, HomeHealthcareServices,
#                                  HospiceCareAmbulatory)

ALL_ENCOUNTERS = reduce(lambda x, y: x | y,
    [c[1] for c in inspect.getmembers(encounter, inspect.isclass) if issubclass(c[1], ValueSet)]
)


class Patient:
    """
    Wrapper around a Canvas patient to add convenience methods specific to science functionality.
    """
    SEX_FEMALE = 'F'
    SEX_MALE = 'M'

    RECORDSET_CLASSES = (
        AdministrativeDocumentRecordSet,
        AllergyIntoleranceRecordSet,
        BillingLineItemRecordSet,
        ConditionRecordSet,
        ImagingReportRecordSet,
        ImmunizationRecordSet,
        InpatientStayRecordSet,
        InstructionRecordSet,
        InterviewRecordSet,
        LabReportRecordSet,
        MedicationRecordSet,
        ProcedureRecordSet,
        ProtocolOverrideRecordSet,
        PrescriptionRecordSet,
        ReasonForVisitRecordSet,
        ReferralRecordSet,
        ReferralReportRecordSet,
        UpcomingAppointmentNoteRecordSet,
        UpcomingAppointmentRecordSet,
        VitalSignRecordSet,
    )

    def __init__(self, data: Dict):
        self.patient = data.get('patient', None)
        self.active_only = False

        self.administrative_documents = AdministrativeDocumentRecordSet.new_from(
            data.get(AdministrativeDocumentRecordSet.API_UPDATE_FIELD, [])
        )

        self.allergy_intolerances = AllergyIntoleranceRecordSet.new_from(
            data.get(AllergyIntoleranceRecordSet.API_UPDATE_FIELD, []))

        self.billing_line_items = BillingLineItemRecordSet.new_from(
            data.get(BillingLineItemRecordSet.API_UPDATE_FIELD, []))

        self.conditions = ConditionRecordSet.new_from(
            data.get(ConditionRecordSet.API_UPDATE_FIELD, []))

        self.imaging_reports = ImagingReportRecordSet.new_from(
            data.get(ImagingReportRecordSet.API_UPDATE_FIELD, []))

        self.immunizations = ImmunizationRecordSet.new_from(
            data.get(ImmunizationRecordSet.API_UPDATE_FIELD, []))

        self.instructions = InstructionRecordSet.new_from(
            data.get(InstructionRecordSet.API_UPDATE_FIELD, []))

        self.interviews = InterviewRecordSet.new_from(
            data.get(InterviewRecordSet.API_UPDATE_FIELD, []))

        self.lab_reports = LabReportRecordSet.new_from(
            data.get(LabReportRecordSet.API_UPDATE_FIELD, []))

        self.medications = MedicationRecordSet.new_from(
            data.get(MedicationRecordSet.API_UPDATE_FIELD, []))

        self.messages = MessageRecordSet.new_from(
            data.get(MessageRecordSet.API_UPDATE_FIELD, []))

        self.procedures = ProcedureRecordSet.new_from(
            data.get(ProcedureRecordSet.API_UPDATE_FIELD, []))

        self.protocol_overrides = ProtocolOverrideRecordSet.new_from(
            data.get(ProtocolOverrideRecordSet.API_UPDATE_FIELD, []))

        self.prescriptions = PrescriptionRecordSet.new_from(
            data.get(PrescriptionRecordSet.API_UPDATE_FIELD, []))

        self.reason_for_visits = ReasonForVisitRecordSet.new_from(
            data.get(ReasonForVisitRecordSet.API_UPDATE_FIELD, []))

        self.referral_reports = ReferralReportRecordSet.new_from(
            data.get(ReferralReportRecordSet.API_UPDATE_FIELD, []))

        self.referrals = ReferralRecordSet.new_from(
            data.get(ReferralRecordSet.API_UPDATE_FIELD, []))

        self.inpatient_stays = InpatientStayRecordSet.new_from(
            data.get(InpatientStayRecordSet.API_UPDATE_FIELD, []))

        self.suspect_hccs = data.get('suspectHccs', [])

        self.upcoming_appointments = UpcomingAppointmentRecordSet.new_from(
            data.get(UpcomingAppointmentRecordSet.API_UPDATE_FIELD))

        self.upcoming_appointment_notes = UpcomingAppointmentNoteRecordSet.new_from(
            data.get(UpcomingAppointmentNoteRecordSet.API_UPDATE_FIELD))

        self.vital_signs = VitalSignRecordSet.new_from(
            data.get(VitalSignRecordSet.API_UPDATE_FIELD, []))

        if self.patient and self.patient.get('addresses'):
            self.addresses = PatientRecordSet.new_from(to_attr_dict(self.patient.get('addresses')))

    @classmethod
    def recordset_fields(cls) -> List[str]:
        """
        Returns available patient record set fields ('conditions', 'medications', etc.)
        """
        return [r_cls.PATIENT_FIELD for r_cls in cls.RECORDSET_CLASSES]

    @property
    def patient_key(self) -> str:
        return self.patient['key']

    @property
    def first_name(self) -> str:
        return self.patient['firstName']

    @property
    def last_name(self) -> str:
        return self.patient.get('lastName', '')

    @property
    def is_female(self) -> bool:
        return self.patient['sexAtBirth'] == self.SEX_FEMALE

    @property
    def is_male(self) -> bool:
        return self.patient['sexAtBirth'] == self.SEX_MALE

    @property
    def is_african_american(self) -> bool:
        return self.patient['biologicalRaceCode'] == '2054-5'

    @property
    def date_of_birth(self) -> str:
        return self.patient['birthDate']

    @property
    def birthday(self) -> arrow.Arrow:
        return arrow.get(self.patient['birthDate'], 'YYYY-MM-DD')

    @property
    def coverages(self) -> List[Dict]:
        return self.patient.get('coverages', [])

    @property
    def age(self):
        return self.age_at(arrow.now())

    # TODO timezones
    def age_at(self, time: arrow.Arrow) -> float:
        """
        Returns the number of years already passed at the time provided
        plus the number of days between the passed birthday and the provided date
        divided by the number of days between the passed and the next birthday
        """
        age = 0
        if not self.patient:
            return age

        birth_day = self.birthday
        if birth_day.date() < time.date():
            age = time.datetime.year - birth_day.datetime.year
            if time.datetime.month < birth_day.datetime.month or (
                    time.datetime.month == birth_day.datetime.month and
                    time.datetime.day < birth_day.datetime.day):
                age -= 1

            current_year = birth_day.shift(years=age)
            next_year = birth_day.shift(years=age + 1)
            age += (time.date() - current_year.date()) / (next_year.date() - current_year.date())
        return age

    def age_at_between(self, time: arrow.Arrow, age_start_inclusive: int,
                       age_end_exclusive: int) -> bool:
        return age_start_inclusive <= self.age_at(time) < age_end_exclusive

    # TODO_REPORTING
    def hospice_within(self, timeframe: Timeframe) -> bool:
        # FIX: is this correct, are these hospice value-sets conditions?
        # return (self.conditions
        #         .before(timeframe.end)
        #         .find(HospiceCareAmbulatory |
        #               DischargedToHomeForHospiceCare |
        #               DischargedToHealthCareFacilityForHospiceCare))  # yapf: disable
        return False

    def has_visit_within(self, timeframe: Timeframe,
                         encounters: Optional[Type[ValueSet]] = None) -> bool:

        return bool(self.count_visit_within(timeframe, encounters) > 0)

    def count_visit_within(self, timeframe: Timeframe,
                           encounters: Optional[Type[ValueSet]] = None) -> int:
        if not encounters:
            encounters = ALL_ENCOUNTERS

        recordset = (self
                     .billing_line_items
                     .after(timeframe.start)
                     .before(timeframe.end)
                     .find(encounters))  # yapf: disable

        reason_for_visit_qs = (self.reason_for_visits
            .after(timeframe.start)
            .before(timeframe.end)
            .find(encounters))

        num_billing_lines = len(recordset.records) if recordset else 0
        num_reasons_for_visit = len(reason_for_visit_qs.records) if reason_for_visit_qs else 0

        return num_billing_lines + num_reasons_for_visit

    def as_dict(self) -> Dict[str, Any]:
        return {
            'patient_key': self.patient_key,
            'patient': self.patient,
            **{
                cls.PATIENT_FIELD: getattr(self, cls.PATIENT_FIELD).records
                for cls in self.RECORDSET_CLASSES
            },
        }

    def print(self) -> None:
        print(json.dumps(self.as_dict(), indent=2))



from ..value_set import ValueSet


class AcuteInpatient(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts related to acute inpatient visits.

    **Data Element Scope:** This value set may use Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with comprehensive history, evaluation, and management of a patient in an acute inpatient setting. This is a grouping value set of CPT and SNOMED codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1083'
    VALUE_SET_NAME = 'Acute Inpatient'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99221',
        '99222',
        '99223',
        '99231',
        '99232',
        '99233',
        '99238',
        '99239',
        '99251',
        '99252',
        '99253',
        '99254',
        '99255',
        '99291'
    }

    SNOMEDCT = {
        '112689000',
        '1505002',
        '15584006',
        '183450002',
        '183481006',
        '183487005',
        '183488000',
        '183489008',
        '183491000',
        '183492007',
        '183493002',
        '183494008',
        '183495009',
        '183496005',
        '183497001',
        '183498006',
        '183499003',
        '183500007',
        '183501006',
        '183502004',
        '183503009',
        '183504003',
        '183505002',
        '183506001',
        '183507005',
        '183508000',
        '183509008',
        '183510003',
        '183511004',
        '183512006',
        '235313004',
        '25986004',
        '287927002',
        '304566005',
        '305337004',
        '305338009',
        '305341000',
        '305342007',
        '305350003',
        '305354007',
        '305355008',
        '305356009',
        '305357000',
        '305358005',
        '305359002',
        '305360007',
        '305361006',
        '305362004',
        '305363009',
        '305364003',
        '305365002',
        '305366001',
        '305367005',
        '305368000',
        '305369008',
        '305370009',
        '305371008',
        '305372001',
        '305374000',
        '305375004',
        '305376003',
        '305377007',
        '305378002',
        '305379005',
        '305380008',
        '305382000',
        '305383005',
        '305384004',
        '305385003',
        '305386002',
        '305387006',
        '305388001',
        '305389009',
        '305390000',
        '305391001',
        '305392008',
        '305393003',
        '305394009',
        '305395005',
        '305396006',
        '305397002',
        '305399004',
        '305400006',
        '305401005',
        '305402003',
        '305403008',
        '305404002',
        '305405001',
        '305406000',
        '305407009',
        '305408004',
        '305409007',
        '305410002',
        '305411003',
        '305412005',
        '305413000',
        '305414006',
        '305415007',
        '305416008',
        '305417004',
        '305418009',
        '305419001',
        '305420007',
        '305421006',
        '305422004',
        '305423009',
        '305424003',
        '305425002',
        '305426001',
        '305427005',
        '305428000',
        '305429008',
        '305430003',
        '305431004',
        '305432006',
        '305433001',
        '305434007',
        '305435008',
        '306732000',
        '306803007',
        '306967009',
        '308251003',
        '308252005',
        '308253000',
        '310361003',
        '3241000175106',
        '32485007',
        '373113001',
        '397769005',
        '398162007',
        '405614004',
        '417005',
        '432621000124105',
        '442281000124108',
        '447941000124106',
        '448421000124105',
        '448431000124108',
        '448441000124103',
        '448851000124103',
        '4563007',
        '45702004',
        '47348005',
        '48183000',
        '51032003',
        '51501005',
        '5161006',
        '52748007',
        '60059000',
        '63551005',
        '699124006',
        '70755000',
        '71290004',
        '76193006',
        '76464004',
        '81672003',
        '82942009',
        '8715000'
    }

class CareServicesInLongTermResidentialFacility(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent patients living in assisted living, domiciliary care or rest homes who have had an interaction with a member of their medical team.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with services provided to new and established patients living in assisted living, domiciliary care or rest home. This is a grouping value set of CPT codes.

    **Exclusion Criteria:** Excludes visits in settings other than assisted living, domiciliary care or rest homes.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1014'
    VALUE_SET_NAME = 'Care Services in Long-Term Residential Facility'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99324',
        '99325',
        '99326',
        '99327',
        '99328',
        '99334',
        '99335',
        '99336',
        '99337'
    }

    SNOMEDCT = {
        '209099002',
        '210098006'
    }

class ClinicalOralEvaluation(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent patients who had a clinical oral evaluation.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with periodic, limited (problem focused

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.125.12.1003'
    VALUE_SET_NAME = 'Clinical Oral Evaluation'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CDT = {
        'D0120',
        'D0140',
        'D0145',
        'D0150',
        'D0160',
        'D0170',
        'D0180'
    }

class ContactOrOfficeVisit(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent contact and office visits for new and established patients, and includes in-person, telephone, online, and other visit types related to depression encounters.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with outpatient contact and office visits in which a patient may be evaluated for depression. This groups CPT and HCPCS codes.

    **Exclusion Criteria:** Excludes inpatients for purposes of the index event. The majority of CPT codes are specified for outpatient visit types; however psychiatry and psychotherapy visits can be used in the inpatient setting.
    """

    OID = '2.16.840.1.113762.1.4.1080.5'
    VALUE_SET_NAME = 'Contact or Office Visit'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '90791',
        '90792',
        '90832',
        '90834',
        '90837',
        '99201',
        '99202',
        '99203',
        '99204',
        '99205',
        '99211',
        '99212',
        '99213',
        '99214',
        '99215',
        '99421',
        '99422',
        '99423',
        '99441',
        '99442',
        '99443',
        '99444'
    }

    HCPCSLEVELII = {
        'G0402',
        'G0438',
        'G0439',
        'G2061',
        'G2062',
        'G2063'
    }

class DetoxificationVisit(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent detoxification visits.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying alcohol and drug detoxification. This is a grouping of SNOMED CT codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1059'
    VALUE_SET_NAME = 'Detoxification Visit'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    SNOMEDCT = {
        '182969009',
        '20093000',
        '23915005',
        '414054004',
        '414056002',
        '56876005',
        '61480009',
        '64297001',
        '67516001',
        '87106005'
    }

class DischargeServicesHospitalInpatient(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent inpatient hospital discharge services.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying hospital discharge day management. This is a grouping of CPT codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1007'
    VALUE_SET_NAME = 'Discharge Services - Hospital Inpatient'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99238',
        '99239'
    }

class DischargeServicesHospitalInpatientSameDayDischarge(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent inpatient hospital same day discharge services.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying observation or inpatient care for the evaluation and management of a patient that results in discharge on the same date of admission. This is a grouping of CPT codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1006'
    VALUE_SET_NAME = 'Discharge Services - Hospital Inpatient Same Day Discharge'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99234',
        '99235',
        '99236'
    }

class DischargeServicesNursingFacility(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent patients who have been discharged from a nursing facility.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with discharge from a nursing facility, including a final examination, instructions for continuing care and preparation of discharge records, prescriptions, and referral forms. Discharge services encounters can be less than or over 30 minutes. This is a grouping of CPT codes.

    **Exclusion Criteria:** Excludes discharges from settings other than a nursing facility.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1013'
    VALUE_SET_NAME = 'Discharge Services - Nursing Facility'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99315',
        '99316'
    }

class DischargeServicesNursingFacility_1065(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent patients who have been discharged from a nursing facility.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with discharge from a nursing facility, including a final examination, instructions for continuing care and preparation of discharge records, prescriptions, and referral forms. Discharge services encounters can be less than or over 30 minutes. This is a grouping of CPT codes.

    **Exclusion Criteria:** Excludes discharges from settings other than a nursing facility.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.11.1065'
    VALUE_SET_NAME = 'Discharge Services - Nursing Facility'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99315',
        '99316'
    }

class Ed(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts related to an ED visit.

    **Data Element Scope:** This value set may use Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with comprehensive history, evaluation, and management of a patient in an ED. This is a grouping value set of CPT and SNOMED codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1085'
    VALUE_SET_NAME = 'ED'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99281',
        '99282',
        '99283',
        '99284',
        '99285'
    }

    SNOMEDCT = {
        '4525004'
    }

class EmergencyDepartmentVisit(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent patients who have had an interaction with a member of their medical care team in the emergency department.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with care provided to new and established patients in the emergency department. This is a value set grouping that includes CPT and SNOMED CT codes.

    **Exclusion Criteria:** Excludes services not performed in the emergency department, including critical care and observation services.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1010'
    VALUE_SET_NAME = 'Emergency Department Visit'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99281',
        '99282',
        '99283',
        '99284',
        '99285'
    }

    SNOMEDCT = {
        '4525004'
    }

class FrailtyEncounter(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent nursing care services provided to frail patients.

    **Data Element Scope:** This value set may use Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with nursing care and home care services provided to frail patients. This is a grouping of CPT, HCPCS, and SNOMEDCT codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1088'
    VALUE_SET_NAME = 'Frailty Encounter'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99504',
        '99509'
    }

    HCPCSLEVELII = {
        'G0162',
        'G0299',
        'G0300',
        'G0493',
        'G0494',
        'S0271',
        'S0311',
        'S9123',
        'S9124',
        'T1000',
        'T1001',
        'T1002',
        'T1003',
        'T1004',
        'T1005',
        'T1019',
        'T1020',
        'T1021',
        'T1022',
        'T1030',
        'T1031'
    }

    SNOMEDCT = {
        '413467001'
    }

class HomeHealthcareServices(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent patients who have had a home health visit by a provider for the evaluation or management of a new or existing patient.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with home visits for the evaluation and management of a new or established patient. This is a grouping value set of CPT and SNOMED CT codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1016'
    VALUE_SET_NAME = 'Home Healthcare Services'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99341',
        '99342',
        '99343',
        '99344',
        '99345',
        '99347',
        '99348',
        '99349',
        '99350'
    }

    SNOMEDCT = {
        '185460008',
        '185462000',
        '185466002',
        '185467006',
        '185468001',
        '185470005',
        '225929007',
        '315205008',
        '439708006',
        '698704008',
        '704126008'
    }

class HospitalInpatientVisitInitial(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent inpatient hospital visits.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with initial hospital care for the evaluation and management of a patient. This is a grouping of CPT codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1004'
    VALUE_SET_NAME = 'Hospital Inpatient Visit - Initial'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99221',
        '99222',
        '99223'
    }

class HospitalObservationCareInitial(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent initial inpatient hospital observation care.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with initial observation care for the evaluation and management of a patient. This is a grouping of CPT codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1002'
    VALUE_SET_NAME = 'Hospital Observation Care - Initial'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99218',
        '99219',
        '99220'
    }

class MedicalDisabilityExam(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent work related or medical disability examinations.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with work related or medical disability examinations. This is a grouping of CPT codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.11.1233'
    VALUE_SET_NAME = 'Medical Disability Exam'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99455',
        '99456'
    }

class NonacuteInpatient(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts related to nonacute inpatient visits.

    **Data Element Scope:** This value set may use Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with comprehensive history, evaluation, and management of a patient in a nonacute inpatient setting. This is a grouping value set of CPT and SNOMED codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1084'
    VALUE_SET_NAME = 'Nonacute Inpatient'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99304',
        '99305',
        '99306',
        '99307',
        '99308',
        '99309',
        '99310',
        '99315',
        '99316',
        '99318',
        '99324',
        '99325',
        '99326',
        '99327',
        '99328',
        '99334',
        '99335',
        '99336',
        '99337'
    }

    SNOMEDCT = {
        '112690009',
        '183430001',
        '183921001',
        '304567001',
        '304568006',
        '305336008',
        '305340004',
        '305381007',
        '306804001',
        '36723004',
        '449411000124106',
        '449421000124103',
        '449431000124100'
    }

class NursingFacilityVisit(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent patients who have had an interaction with a member of their medical team on admission to a nursing facility.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with services provided to new and established patients in a nursing facility (skilled, intermediate and long-term care facilities).

    **Exclusion Criteria:** Excludes visits in settings other than a nursing facility.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1012'
    VALUE_SET_NAME = 'Nursing Facility Visit'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99304',
        '99305',
        '99306',
        '99307',
        '99308',
        '99309',
        '99310',
        '99315',
        '99316',
        '99318'
    }

    SNOMEDCT = {
        '18170008',
        '207195004'
    }

class Observation(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts related to observation visits.

    **Data Element Scope:** This value set may use Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with comprehensive history, evaluation, and management of a patient in an observation care setting. This is a grouping value set of CPT codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1086'
    VALUE_SET_NAME = 'Observation'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99217',
        '99218',
        '99219',
        '99220'
    }

class OfficeVisit(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent patients who have had an office or other outpatient visit.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with comprehensive history, evaluation, and management of a patient in an office or outpatient facility. Patient can be presenting with problems that are minor to high severity. This is a grouping value set of CPT codes.

    **Exclusion Criteria:** Excludes non-office visits, including telehealth services.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1001'
    VALUE_SET_NAME = 'Office Visit'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99201',
        '99202',
        '99203',
        '99204',
        '99205',
        '99212',
        '99213',
        '99214',
        '99215'
    }

    SNOMEDCT = {
        '185463005',
        '185464004',
        '185465003',
        '30346009',
        '3391000175108',
        '37894004',
        '439740005'
    }

class Outpatient(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts related to outpatient visits.

    **Data Element Scope:** This value set may use Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with comprehensive history, evaluation, and management of a patient in an outpatient setting. This is a grouping value set of CPT and HCPCS codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1087'
    VALUE_SET_NAME = 'Outpatient'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99201',
        '99202',
        '99203',
        '99204',
        '99205',
        '99211',
        '99212',
        '99213',
        '99214',
        '99215',
        '99241',
        '99242',
        '99243',
        '99244',
        '99245',
        '99341',
        '99342',
        '99343',
        '99344',
        '99345',
        '99347',
        '99348',
        '99349',
        '99350',
        '99381',
        '99382',
        '99383',
        '99384',
        '99385',
        '99386',
        '99387',
        '99391',
        '99392',
        '99393',
        '99394',
        '99395',
        '99396',
        '99397',
        '99401',
        '99402',
        '99403',
        '99404',
        '99411',
        '99412',
        '99429',
        '99455',
        '99456',
        '99483'
    }

    HCPCSLEVELII = {
        'G0402',
        'G0438',
        'G0439',
        'G0463',
        'T1015'
    }

class OutpatientConsultation(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent patients who have had an outpatient interaction at an office with a member of their medical care team.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with comprehensive history, evaluation, and management of a patient in an office or outpatient facility. Patient can be presenting with problems that are minor to high severity. This is a grouping value set of CPT and SNOMED CT codes.

    **Exclusion Criteria:** Excludes non-office visits, including telehealth services.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1008'
    VALUE_SET_NAME = 'Outpatient Consultation'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99241',
        '99242',
        '99243',
        '99244',
        '99245'
    }

    SNOMEDCT = {
        '281036007',
        '77406008'
    }

class PreventiveCareEstablishedOfficeVisit0To17(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent initial comprehensive preventive medical evaluation, including regular preventive care or care of small problem or preexisting condition that requires no extra work, to be associated with patients 0-17 years of age, that received prior outpatient professional services from the physician practice in the last 3 years.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with ages 0-17, and indicating initial comprehensive preventive medical evaluation, including regular preventive care or care of small problem or preexisting condition that requires no extra work, for a patient that received prior outpatient professional services from the physician practice in the last 3 years. This is a grouping value set of CPT codes.

    **Exclusion Criteria:** Excludes codes that are not for comprehensive preventive medical evaluations and codes that are for patients who have not been seen in the last 3 years.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1024'
    VALUE_SET_NAME = 'Preventive Care, Established Office Visit, 0 to 17'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99391',
        '99392',
        '99393',
        '99394'
    }

class PreventiveCareServicesEstablishedOfficeVisit18AndUp(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent patients over the age of 18 who have had an established preventive care office visit.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with comprehensive preventive medicine reevaluation and management of an individual the age of 18 years or over. This is a grouping value set of CPT codes.

    **Exclusion Criteria:** Excludes non-office visits, including telehealth services.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1025'
    VALUE_SET_NAME = 'Preventive Care Services - Established Office Visit, 18 and Up'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99395',
        '99396',
        '99397'
    }

class PreventiveCareServicesGroupCounseling(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent group counseling services.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying preventive medicine counseling and/or risk factor reduction intervention(s) provided to individuals in a group setting.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1027'
    VALUE_SET_NAME = 'Preventive Care Services - Group Counseling'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99411',
        '99412'
    }

class PreventiveCareServicesIndividualCounseling(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent patients who have received preventive medicine counseling and/or risk factor reduction interventions.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with counseling, anticipatory guidance, and risk factor reduction interventions. Preventative care and individual counseling encounters can be 15 to 60 minutes. This is a grouping of CPT codes.

    **Exclusion Criteria:** Excludes codes for services performed in the emergency department, including critical care and observation services.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1026'
    VALUE_SET_NAME = 'Preventive Care Services-Individual Counseling'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99401',
        '99402',
        '99403',
        '99404'
    }

class PreventiveCareServicesInitialOfficeVisit0To17(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent initial comprehensive preventive medical evaluation, including regular preventive care or care of small problem or preexisting condition that requires no extra work, to be associated with patients 0-17 years of age, that have no prior outpatient professional services from the physician practice in the last 3 years.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with ages 0-17, and that indicate initial comprehensive preventive medical evaluation, including regular preventive care or care of small problem or preexisting condition that requires no extra work, for a patient that has no prior outpatient professional services from the physician practice in the last 3 years. This is a grouping value set of CPT codes.

    **Exclusion Criteria:** Excludes codes that are not for comprehensive preventive medical evaluations and codes that are for patients who have been seen in the last 3 years.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1022'
    VALUE_SET_NAME = 'Preventive Care Services, Initial Office Visit, 0 to 17'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99381',
        '99382',
        '99383',
        '99384'
    }

class PreventiveCareServicesInitialOfficeVisit18AndUp(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent patients over the age of 18 who have had an initial preventive care office visit.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with comprehensive preventive medicine reevaluation and management of an individual the age of 18 years or over. This is a grouping value set of CPT codes.

    **Exclusion Criteria:** Excludes non-office visits, including telehealth services.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1023'
    VALUE_SET_NAME = 'Preventive Care Services-Initial Office Visit, 18 and Up'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99385',
        '99386',
        '99387'
    }

class PreventiveCareServicesOther(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent unlisted preventive medicine services.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with unlisted preventive medicine services. This is a grouping of a CPT codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1030'
    VALUE_SET_NAME = 'Preventive Care Services - Other'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99429'
    }

class PsychotherapyAndPharmacologicManagement(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent psychotherapy services.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying psychotherapy services.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1055'
    VALUE_SET_NAME = 'Psychotherapy and Pharmacologic Management'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '90845',
        '90847',
        '90849',
        '90853',
        '90875',
        '90876'
    }

class TelehealthServices(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent telehealth services.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying telehealth services, including telephone and online evaluation and management services.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1031'
    VALUE_SET_NAME = 'Telehealth Services'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '98966',
        '98967',
        '98968',
        '99441',
        '99442',
        '99443'
    }

class TelephoneEvaluation(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent telephone evaluations.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying evaluation and management services to a patient by telephone. This is a grouping of CPT codes.

    **Exclusion Criteria:** Excludes telephone evaluation and management services that last for less than five minutes.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1082'
    VALUE_SET_NAME = 'Telephone Evaluation'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99441',
        '99442',
        '99443'
    }

class TelephoneManagement(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent telephone management.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with all relevant codes used to identify assessment and management services to a patient by telephone. This is a grouping of CPT codes.

    **Exclusion Criteria:** Excludes telephone assessment and management services that last for less than five minutes.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1053'
    VALUE_SET_NAME = 'Telephone Management'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '98966',
        '98967',
        '98968'
    }

class TelephoneVisits(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent telephone visits.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Encounter, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying assessment, evaluation and management services to a patient by telephone. This is a grouping of CPT codes.

    **Exclusion Criteria:** Excludes telephone assessment, evaluation and management services that last for less than five minutes.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1080'
    VALUE_SET_NAME = 'Telephone Visits'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '98966',
        '98967',
        '98968',
        '99441',
        '99442',
        '99443'
    }


from ..value_set import ValueSet


class AlcoholAndDrugDependenceTreatment(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent treatment for alcohol and drug dependence.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with assessment, management, and both psychosocial and pharmacological treatment of alcohol and drug dependence. This is a grouping of SNOMED CT codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.106.12.1005'
    VALUE_SET_NAME = 'Alcohol and Drug Dependence Treatment'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    SNOMEDCT = {
        '171047005',
        '20093000',
        '23915005',
        '24165007',
        '266707007',
        '310653000',
        '313071005',
        '370881007',
        '370884004',
        '385989002',
        '386448003',
        '386449006',
        '386450006',
        '386451005',
        '408933008',
        '408934002',
        '408935001',
        '408936000',
        '408941008',
        '408942001',
        '408943006',
        '408944000',
        '408945004',
        '408947007',
        '408948002',
        '410419007',
        '413473000',
        '423416000',
        '424148004',
        '424407005',
        '424589009',
        '426928008',
        '429291000124102',
        '56876005',
        '60112009',
        '707166002',
        '720174008',
        '720175009',
        '720176005',
        '720177001',
        '737363002'
    }

class Ambulatory(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent encounters in an ambulatory setting.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category or attribute related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with care in an ambulatory setting.

    **Exclusion Criteria:** Excludes patients who had ambulatory hospice care.
    """

    OID = '2.16.840.1.113883.3.464.1003.122.12.1003'
    VALUE_SET_NAME = 'Ambulatory'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    SNOMEDCT = {
        '255327002',
        '440655000',
        '722171005'
    }

class AnnualWellnessVisit(ValueSet):
    """
    **Clinical Focus:** This value set grouping contains concepts that represent annual wellness visits.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter. The intent of this data element is to identify patients who have had an annual wellness visit.

    **Inclusion Criteria:** Includes only relevant concepts associated with encounters specific to annual wellness visits.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.526.3.1240'
    VALUE_SET_NAME = 'Annual Wellness Visit'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    HCPCSLEVELII = {
        'G0438',
        'G0439'
    }

    SNOMEDCT = {
        '444971000124105',
        '456201000124103'
    }

class AudiologyVisit(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent assessment tools or instruments used to quantify hearing.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with hearing assessment tools and instruments used to quantify hearing such as basic vestibular evaluation, spontaneous and positional nystagmus testing, and computerized dynamic posturography. This is a grouping of CPT codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1066'
    VALUE_SET_NAME = 'Audiology Visit'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '92540',
        '92541',
        '92542',
        '92548'
    }

class BehavioralHealthFollowUpVisit(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent patients who have had a behavioral health follow-up visit.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with health and behavior assessment or intervention, or education and training for patient self-management by a physician or other qualified health care professional.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.12.1054'
    VALUE_SET_NAME = 'Behavioral Health Follow-up Visit'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '98960',
        '98961',
        '98962',
        '99078',
        '99510'
    }

class BehavioralneuropsychAssessment(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent visits for assessments for neuropsychological assessments.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with applicable neuropsychological assessments for behavioral health.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.526.3.1023'
    VALUE_SET_NAME = 'Behavioral/Neuropsych Assessment'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '96116'
    }

    SNOMEDCT = {
        '307808008'
    }

class EncounterInfluenza(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent visits during which an influenza vaccine may be administered.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with face-to-face encounters during which an influenza vaccine may be administered.

    **Exclusion Criteria:** Excludes telehealth encounters.
    """

    OID = '2.16.840.1.113883.3.526.3.1252'
    VALUE_SET_NAME = 'Encounter-Influenza'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '99201',
        '99202',
        '99203',
        '99204',
        '99205',
        '99212',
        '99213',
        '99214',
        '99215',
        '99241',
        '99242',
        '99243',
        '99244',
        '99245',
        '99304',
        '99305',
        '99306',
        '99307',
        '99308',
        '99309',
        '99310',
        '99315',
        '99316',
        '99318',
        '99324',
        '99325',
        '99326',
        '99327',
        '99328',
        '99334',
        '99335',
        '99336',
        '99337',
        '99341',
        '99342',
        '99343',
        '99344',
        '99345',
        '99347',
        '99348',
        '99349',
        '99350',
        '99381',
        '99382',
        '99383',
        '99384',
        '99385',
        '99386',
        '99387',
        '99391',
        '99392',
        '99393',
        '99394',
        '99395',
        '99396',
        '99397',
        '99401',
        '99402',
        '99403',
        '99404',
        '99411',
        '99412',
        '99429'
    }

    HCPCSLEVELII = {
        'G0438',
        'G0439'
    }

    SNOMEDCT = {
        '18170008',
        '185460008',
        '185462000',
        '185463005',
        '185464004',
        '185465003',
        '185466002',
        '185467006',
        '185468001',
        '185470005',
        '207195004',
        '209099002',
        '210098006',
        '225929007',
        '281036007',
        '30346009',
        '315205008',
        '3391000175108',
        '37894004',
        '439708006',
        '439740005',
        '698704008',
        '704126008',
        '77406008'
    }

class EncounterInpatient(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent the most common inpatient encounter types.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter. The intent of this data element is to identify patients who have had an inpatient encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with SNOMED CT codes representing inpatient encounter.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.666.5.307'
    VALUE_SET_NAME = 'Encounter Inpatient'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    SNOMEDCT = {
        '183452005',
        '32485007',
        '8715000'
    }

class EncounterToDocumentMedications(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent multiple types of encounters that include documentation of current medications.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category or attribute related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying types of encounters associated with documentation of medications reviewed, updated or transcribed as the current medication list.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.600.1.1834'
    VALUE_SET_NAME = 'Encounter to Document Medications'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '59400',
        '59510',
        '59610',
        '59618',
        '90791',
        '90792',
        '90832',
        '90834',
        '90837',
        '90839',
        '92002',
        '92004',
        '92012',
        '92014',
        '92507',
        '92508',
        '92526',
        '92537',
        '92538',
        '92540',
        '92541',
        '92542',
        '92544',
        '92545',
        '92547',
        '92548',
        '92550',
        '92557',
        '92567',
        '92568',
        '92570',
        '92585',
        '92588',
        '92626',
        '96116',
        '96156',
        '96158',
        '97129',
        '97161',
        '97162',
        '97163',
        '97164',
        '97165',
        '97166',
        '97167',
        '97168',
        '97802',
        '97803',
        '97804',
        '98960',
        '98961',
        '98962',
        '99201',
        '99202',
        '99203',
        '99204',
        '99205',
        '99212',
        '99213',
        '99214',
        '99215',
        '99221',
        '99222',
        '99223',
        '99236',
        '99281',
        '99282',
        '99283',
        '99284',
        '99285',
        '99304',
        '99305',
        '99306',
        '99307',
        '99308',
        '99309',
        '99310',
        '99315',
        '99316',
        '99318',
        '99324',
        '99325',
        '99326',
        '99327',
        '99328',
        '99334',
        '99335',
        '99336',
        '99337',
        '99339',
        '99340',
        '99341',
        '99342',
        '99343',
        '99344',
        '99345',
        '99347',
        '99348',
        '99349',
        '99350',
        '99385',
        '99386',
        '99387',
        '99395',
        '99396',
        '99397',
        '99495',
        '99496'
    }

    HCPCSLEVELII = {
        'G0101',
        'G0108',
        'G0270',
        'G0402',
        'G0438',
        'G0439'
    }

    SNOMEDCT = {
        '10197000',
        '108220007',
        '108221006',
        '108224003',
        '108311000',
        '13607009',
        '14736009',
        '165171009',
        '18091003',
        '18512000',
        '185349003',
        '185463005',
        '185465003',
        '209099002',
        '210098006',
        '225967005',
        '252592009',
        '252624005',
        '270427003',
        '270430005',
        '273643004',
        '274803000',
        '277404009',
        '284015009',
        '308335008',
        '34651001',
        '35025007',
        '36228007',
        '370803007',
        '385973000',
        '386372009',
        '390906007',
        '405096004',
        '406547006',
        '408983003',
        '410155007',
        '410157004',
        '410158009',
        '410160006',
        '410170008',
        '439708006',
        '439952009',
        '440524004',
        '46662001',
        '48423005',
        '50357006',
        '53555003',
        '54290001',
        '63547008',
        '66902005',
        '78318003',
        '83607001',
        '8411005',
        '86013001',
        '90526000',
        '91573000'
    }

class EncounterToEvaluateBmi(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent multiple types of encounters for body mass index (BMI) assessment.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category or attribute related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with multiple types of encounters that enable BMI measurement.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.600.1.1751'
    VALUE_SET_NAME = 'Encounter to Evaluate BMI'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CDT = {
        'D7111',
        'D7140',
        'D7210',
        'D7220',
        'D7230',
        'D7240',
        'D7241',
        'D7250',
        'D7251'
    }

    CPT = {
        '90791',
        '90792',
        '90832',
        '90834',
        '90837',
        '96156',
        '96158',
        '96159',
        '97161',
        '97162',
        '97163',
        '97165',
        '97166',
        '97167',
        '97802',
        '97803',
        '99201',
        '99202',
        '99203',
        '99204',
        '99205',
        '99212',
        '99213',
        '99214',
        '99215',
        '99236',
        '99304',
        '99305',
        '99306',
        '99307',
        '99308',
        '99309',
        '99310',
        '99315',
        '99316',
        '99318',
        '99324',
        '99325',
        '99326',
        '99327',
        '99328',
        '99334',
        '99335',
        '99336',
        '99337',
        '99339',
        '99340',
        '99385',
        '99386',
        '99387',
        '99395',
        '99396',
        '99397',
        '99401',
        '99402'
    }

    HCPCSLEVELII = {
        'G0101',
        'G0108',
        'G0270',
        'G0271',
        'G0402',
        'G0438',
        'G0439',
        'G0447',
        'G0473'
    }

    SNOMEDCT = {
        '10197000',
        '108220007',
        '108221006',
        '108224003',
        '108311000',
        '13607009',
        '14736009',
        '165171009',
        '18512000',
        '185349003',
        '185463005',
        '185465003',
        '225967005',
        '270427003',
        '270430005',
        '308335008',
        '35025007',
        '386372009',
        '390906007',
        '406547006',
        '410155007',
        '410157004',
        '410158009',
        '410160006',
        '410170008',
        '410172000',
        '46662001',
        '55162003',
        '68381003',
        '78318003',
        '83607001',
        '8411005',
        '86013001',
        '90526000'
    }

class EncounterToScreenForBloodPressure(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent multiple types of encounters where blood pressure measurement screening can be performed.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with encounters where the actual procedure of taking a blood pressure is performed.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.600.1920'
    VALUE_SET_NAME = 'Encounter to Screen for Blood Pressure'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CDT = {
        'D7111',
        'D7140',
        'D7210',
        'D7220',
        'D7230',
        'D7240',
        'D7241',
        'D7250',
        'D7251'
    }

    CPT = {
        '90791',
        '90792',
        '92002',
        '92004',
        '92012',
        '92014',
        '99201',
        '99202',
        '99203',
        '99204',
        '99205',
        '99212',
        '99213',
        '99214',
        '99215',
        '99236',
        '99281',
        '99282',
        '99283',
        '99284',
        '99285',
        '99304',
        '99305',
        '99306',
        '99307',
        '99308',
        '99309',
        '99310',
        '99315',
        '99316',
        '99318',
        '99324',
        '99325',
        '99326',
        '99327',
        '99328',
        '99334',
        '99335',
        '99336',
        '99337',
        '99339',
        '99340',
        '99341',
        '99342',
        '99343',
        '99344',
        '99345',
        '99347',
        '99348',
        '99349',
        '99350',
        '99385',
        '99386',
        '99387',
        '99395',
        '99396',
        '99397'
    }

    HCPCSLEVELII = {
        'G0101',
        'G0402',
        'G0438',
        'G0439'
    }

    SNOMEDCT = {
        '103705002',
        '108220007',
        '108221006',
        '108224003',
        '12843005',
        '13607009',
        '14736009',
        '18170008',
        '185349003',
        '185463005',
        '185465003',
        '207195004',
        '209099002',
        '210098006',
        '270427003',
        '270430005',
        '306677009',
        '308335008',
        '35025007',
        '390906007',
        '406547006',
        '439708006',
        '4525004',
        '46662001',
        '50357006',
        '76464004',
        '78318003',
        '83607001',
        '86013001',
        '90526000'
    }

class EncounterToScreenForDepression(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts in which a depression screen could be assessed and documented through an exam, assessment, interview or evaluation.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category or attribute related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with wellness visits, annual visits, therapy evaluations, or primary or specialist physician office visits where a depression screen could be conducted.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.600.1916'
    VALUE_SET_NAME = 'Encounter to Screen for Depression'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '59400',
        '59510',
        '59610',
        '59618',
        '90791',
        '90792',
        '90832',
        '90834',
        '90837',
        '92625',
        '96105',
        '96110',
        '96112',
        '96116',
        '96125',
        '96136',
        '96138',
        '96156',
        '96158',
        '97165',
        '97166',
        '97167',
        '99078',
        '99201',
        '99202',
        '99203',
        '99204',
        '99205',
        '99212',
        '99213',
        '99214',
        '99215',
        '99304',
        '99305',
        '99306',
        '99307',
        '99308',
        '99309',
        '99310',
        '99315',
        '99316',
        '99318',
        '99324',
        '99325',
        '99326',
        '99327',
        '99328',
        '99334',
        '99335',
        '99336',
        '99337',
        '99339',
        '99340',
        '99384',
        '99385',
        '99386',
        '99387',
        '99394',
        '99395',
        '99396',
        '99397',
        '99401',
        '99402',
        '99403',
        '99404',
        '99483',
        '99484',
        '99492',
        '99493'
    }

    HCPCSLEVELII = {
        'G0101',
        'G0402',
        'G0438',
        'G0439',
        'G0444'
    }

    SNOMEDCT = {
        '10197000',
        '108220007',
        '108221006',
        '108224003',
        '108311000',
        '13607009',
        '14736009',
        '165171009',
        '171207006',
        '18512000',
        '185349003',
        '185463005',
        '185465003',
        '252603000',
        '270427003',
        '270430005',
        '302440009',
        '308335008',
        '35025007',
        '370803007',
        '390906007',
        '406547006',
        '410155007',
        '410157004',
        '46662001',
        '53555003',
        '78318003',
        '83607001',
        '8411005',
        '86013001',
        '90526000'
    }

class EsrdMonthlyOutpatientServices(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent patients who had ESRD monthly outpatient services.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying patients who had ESRD monthly outpatients services.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.109.12.1014'
    VALUE_SET_NAME = 'ESRD Monthly Outpatient Services'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '90951',
        '90952',
        '90953',
        '90954',
        '90955',
        '90956',
        '90957',
        '90958',
        '90959',
        '90960',
        '90961',
        '90962',
        '90963',
        '90964',
        '90965',
        '90966',
        '90967',
        '90968',
        '90969',
        '90970',
        '90989',
        '90993',
        '90997',
        '90999',
        '99512'
    }

class FollowUpWithin4Weeks(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent capturing the follow-up encounter within four weeks of the initial encounter.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter. Intention to represent follow-up visits within a 4-week period.

    **Inclusion Criteria:** Includes only relevant concepts associated with capturing the follow-up between 1 day to 1 month of the initial finding.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.526.3.1578'
    VALUE_SET_NAME = 'Follow Up Within 4 Weeks'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    SNOMEDCT = {
        '183617005',
        '183618000',
        '183619008',
        '183620002',
        '183621003',
        '183622005',
        '183623000'
    }

class FollowUpWithinOneYear(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent capturing the follow-up encounter within one year of the initial encounter.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter. Intention to represent follow-up visits within a one-year period.

    **Inclusion Criteria:** Includes only relevant concepts associated with capturing the follow-up between 1 day to 12 month of the initial finding.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.526.3.1579'
    VALUE_SET_NAME = 'Follow Up Within One Year'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    SNOMEDCT = {
        '183617005',
        '183618000',
        '183619008',
        '183620002',
        '183621003',
        '183622005',
        '183623000',
        '183624006',
        '183625007',
        '183626008',
        '183627004',
        '183628009',
        '300042001'
    }

class GroupPsychotherapy(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent visits during which group psychotherapy is performed.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with group psychotherapy.

    **Exclusion Criteria:** Excludes family psychotherapy or individual psychotherapy.
    """

    OID = '2.16.840.1.113883.3.526.3.1187'
    VALUE_SET_NAME = 'Group Psychotherapy'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '90853'
    }

    SNOMEDCT = {
        '76168009'
    }

class OccupationalTherapyEvaluation(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent visits for evaluations for occupational therapy services.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with applicable codes for occupational therapy evaluations.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.526.3.1011'
    VALUE_SET_NAME = 'Occupational Therapy Evaluation'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '97165',
        '97166',
        '97167',
        '97168'
    }

    SNOMEDCT = {
        '410155007',
        '410157004'
    }

class OphthalmologicServices(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent ophthalmological visits.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with relevant encounters during which the patient sees an eye care professional.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.101.11.1206'
    VALUE_SET_NAME = 'Ophthalmologic Services'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '92002',
        '92004',
        '92012',
        '92014'
    }

class OphthalmologicalServices(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent ophthalmological visits.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with encounters during which the patient sees an eye care professional.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.526.3.1285'
    VALUE_SET_NAME = 'Ophthalmological Services'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '92002',
        '92004',
        '92012',
        '92014'
    }

    SNOMEDCT = {
        '359960003',
        '36228007',
        '66902005',
        '78831002'
    }

class OutpatientEncountersForPreventiveCare(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent outpatient encounters.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category or attribute related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with evaluation and management of a new or established patient through an encounter for annual visit, preventive evaluation, follow-up, or periodic re-evaluations that would occur in the outpatient setting.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.526.3.1576'
    VALUE_SET_NAME = 'Outpatient Encounters for Preventive Care'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    SNOMEDCT = {
        '108219001',
        '108220007',
        '108221006',
        '108224003',
        '14736009',
        '185349003',
        '185389009',
        '270427003',
        '270430005',
        '281036007',
        '308335008',
        '390906007',
        '410187005',
        '78318003',
        '86013001',
        '90526000'
    }

class PalliativeCareEncounter(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent identifying patients receiving palliative, comfort or hospice care.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying patients receiving palliative, comfort or hospice care.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.600.1.1575'
    VALUE_SET_NAME = 'Palliative Care Encounter'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    ICD10CM = {
        'Z515'
    }

class PatientProviderInteraction(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent provider interactions with patients that include both face-to-face and virtual types of encounters.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with provider interactions with patients that are conducted in an office or care setting, as well as interactions which may occur via virtual methods, such as telephone calls, emails, and letters.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.526.3.1012'
    VALUE_SET_NAME = 'Patient Provider Interaction'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    SNOMEDCT = {
        '185316007',
        '185317003',
        '185318008',
        '185320006',
        '185321005',
        '185349003',
        '185463005',
        '185465003',
        '270424005',
        '270427003',
        '270430005',
        '308335008',
        '308720009',
        '386473003',
        '390906007',
        '401267002',
        '401271004',
        '406547006',
        '438515009',
        '438516005',
        '445450000',
        '448337001',
        '87790002',
        '90526000'
    }

class PhysicalTherapyEvaluation(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent visits for evaluations for physical therapy services.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with applicable codes for physical therapy evaluations.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.526.3.1022'
    VALUE_SET_NAME = 'Physical Therapy Evaluation'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '97161',
        '97162',
        '97163'
    }

    SNOMEDCT = {
        '33849009'
    }

class PsychVisitDiagnosticEvaluation(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent visits during which a psychiatric diagnostic evaluation is completed.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with visits with the completion of psychiatric diagnostic evaluations.

    **Exclusion Criteria:** Excludes group psychotherapy or family psychotherapy.
    """

    OID = '2.16.840.1.113883.3.526.3.1492'
    VALUE_SET_NAME = 'Psych Visit - Diagnostic Evaluation'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '90791',
        '90792'
    }

    SNOMEDCT = {
        '10197000',
        '165172002',
        '68338001',
        '79094001'
    }

class PsychVisitFamilyPsychotherapy(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent visits during which family psychotherapy is performed.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with family psychotherapy.

    **Exclusion Criteria:** Excludes group psychotherapy or individual psychotherapy visits.
    """

    OID = '2.16.840.1.113883.3.526.3.1018'
    VALUE_SET_NAME = 'Psych Visit - Family Psychotherapy'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '90846',
        '90847'
    }

    SNOMEDCT = {
        '108313002'
    }

class PsychVisitPsychotherapy(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent visits for psychotherapy.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with visits for individual psychotherapy services.

    **Exclusion Criteria:** Excludes group psychotherapy or family psychotherapy.
    """

    OID = '2.16.840.1.113883.3.526.3.1496'
    VALUE_SET_NAME = 'Psych Visit - Psychotherapy'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '90832',
        '90834',
        '90837'
    }

    SNOMEDCT = {
        '183381005',
        '183382003',
        '183383008',
        '18512000',
        '302242004',
        '304820009',
        '304822001',
        '314034001',
        '38678006',
        '401157001',
        '443730003',
        '75516001',
        '90102008'
    }

class Psychoanalysis(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent visits for psychoanalysis.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with psychoanalysis.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.526.3.1141'
    VALUE_SET_NAME = 'Psychoanalysis'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '90845'
    }

    SNOMEDCT = {
        '28988002',
        '61436009'
    }

class SpeechAndHearingEvaluation(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent visits for speech and hearing evaluation.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Encounter.

    **Inclusion Criteria:** Includes only relevant concepts associated with evaluation of speech and/or hearing related abilities.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.526.3.1530'
    VALUE_SET_NAME = 'Speech and Hearing Evaluation'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    CPT = {
        '92521',
        '92522',
        '92523',
        '92524',
        '92540',
        '92557',
        '92625'
    }

    SNOMEDCT = {
        '26342005',
        '41375007',
        '77837000',
        '91515000'
    }


import arrow

from canvas_workflow_kit.timeframe import Timeframe
from canvas_workflow_kit.value_set.v2018 import TobaccoUseScreening, AdultDepressionScreening

from .base import SDKBaseTest


class SDKInterviewTest(SDKBaseTest):

    def test_interviews(self):
        patient = self.load_patient('example')
        self.assertTrue(patient)

        # look for interview he doesn't have
        t = Timeframe(arrow.get('2017-01-01'), arrow.get('2018-02-01'))
        self.assertFalse(patient.interviews.find(AdultDepressionScreening).within(t).last())

        # look for an interview he does have
        self.assertTrue(patient.interviews.find(TobaccoUseScreening).within(t).last())



import arrow

from canvas_workflow_kit import settings

from canvas_workflow_kit.builtin_cqms.hcc001v1_problem_list_hygiene import Hcc001v1
from canvas_workflow_kit.patient_recordset import ConditionRecordSet, ProtocolOverrideRecordSet
from canvas_workflow_kit.tests.base import SDKBaseTest
from canvas_workflow_kit.timeframe import Timeframe


class TestCanvasHcc001v1(SDKBaseTest):

    def setUp(self):
        self.mocks_path = f'{settings.BASE_DIR}/builtin_cqms/tests/mock_data'

    def test_enabled(self):
        self.assertTrue(Hcc001v1.enabled())

    def test_description(self):
        expected = ('All patients with active condition not assessed within the last year.')
        self.assertEqual(expected, Hcc001v1._meta.description)

    def test_information(self):
        expected = ('https://canvas-medical.zendesk.com/hc/en-us/articles/360059083693-Problem-List-Hygiene-HCC001v1')
        self.assertEqual(expected, Hcc001v1._meta.information)

    def test_change_types(self):
        result = Hcc001v1._meta.compute_on_change_types
        expected = [
            'protocol_override',
            'condition',
        ]
        self.assertEqual(len(expected), len(result))
        for change in expected:
            self.assertIn(change, result)

    def test_active_hcc(self):
        start = arrow.get('2017-08-23 13:24:56')
        end = arrow.get('2018-08-23 13:24:56')
        timeframe = Timeframe(start=start, end=end)

        # patient has a active condition in HCC
        patient = self.load_patient('hcc001v1')
        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        result = tested.active_hcc

        self.assertEqual(1, len(result))
        self.assertEqual('I429', result[0]['ICD10'])
        self.assertEqual('2018-08-22 23:08:00', result[0]['date'].format('YYYY-MM-DD HH:MM:SS'))

        patient.conditions = patient.conditions + patient.conditions + patient.conditions
        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        result = tested.active_hcc

        self.assertEqual(3, len(result))

        # patient has NO active condition in HCC
        patient = self.load_patient('hcc001v1')
        patient.conditions[0]['clinicalStatus'] = 'XXXXXXXX'
        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        result = tested.active_hcc

        self.assertEqual(0, len(result))

        # patient has a condition NOT in HCC
        patient = self.load_patient('hcc001v1')
        patient.conditions[0]['coding'][0]['code'] = 'XXXXXXXX'
        patient.conditions[0]['coding'][1]['code'] = 'XXXXXXXX'
        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        result = tested.active_hcc

        self.assertEqual(0, len(result))

        # patient has NO condition
        patient = self.load_patient('hcc001v1')
        patient.conditions = ConditionRecordSet([])
        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        result = tested.active_hcc

        self.assertEqual(0, len(result))

    def test_too_old_hccs(self):
        patient = self.load_patient('hcc001v1')
        patient.conditions = ConditionRecordSet([
            {
                'coding': [{'system': 'ICD-10', 'code': 'I427', }],
                'clinicalStatus': 'active',
                'periods': [{'from': '2018-05-22', 'to': None}],
                'lastTimestamps': {'assessed': '2017-08-22 13:24:56', },
            },
            {
                'coding': [{'system': 'ICD-10', 'code': 'I428', }],
                'clinicalStatus': 'active',
                'periods': [{'from': '2018-05-22', 'to': None}],
                'lastTimestamps': {'assessed': '2017-07-21 13:24:56', },
            },
            {
                'coding': [{'system': 'ICD-10', 'code': 'I429', }],
                'clinicalStatus': 'active',
                'periods': [{'from': '2018-05-22', 'to': None}],
                'lastTimestamps': {'assessed': '2017-09-22 13:24:56', },
            },
        ])  # yapf: disable

        # no adjustment
        start = arrow.get('2017-08-23 13:24:56')
        end = arrow.get('2018-08-23 13:24:56')
        timeframe = Timeframe(start=start, end=end)

        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        result = tested.too_old_hccs
        expected = [
            'I427',
            'I428',
        ]
        self.assertEqual(len(expected), len(result))
        for item in result:
            self.assertIn(item['ICD10'], expected)

        # with adjustment
        patient.protocol_overrides = ProtocolOverrideRecordSet([{
            'protocolKey': 'Hcc001v1',
            'adjustment': {
                'reference': '2018-02-01T00:00:00Z',
                'cycleDays': 180
            },
            'snooze': None,
            'modified': '2018-03-15T12:32:54Z',
        }])  # yapf: disable
        tested = Hcc001v1(patient=patient, now=arrow.get('2018-02-01').shift(days=20))
        result = tested.too_old_hccs
        expected = [
            'I427',
            'I428',
        ]
        self.assertEqual(len(expected), len(result))
        for item in result:
            self.assertIn(item['ICD10'], expected)
        tested = Hcc001v1(patient=patient, now=arrow.get('2018-02-01').shift(days=80))
        result = tested.too_old_hccs
        expected = [
            'I427',
            'I428',
            'I429',
        ]
        self.assertEqual(len(expected), len(result))
        for item in result:
            self.assertIn(item['ICD10'], expected)

    def test_in_initial_population(self):
        start = arrow.get('2017-08-23 13:24:56')
        end = arrow.get('2018-08-23 13:24:56')
        timeframe = Timeframe(start=start, end=end)

        # any patient --> true
        patient = self.load_patient('hcc001v1')
        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        self.assertTrue(tested.in_initial_population())

        patient.conditions = []
        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        self.assertTrue(tested.in_initial_population())

    def test_in_denominator(self):
        start = arrow.get('2017-08-23 13:24:56')
        end = arrow.get('2018-08-23 13:24:56')
        timeframe = Timeframe(start=start, end=end)

        # patient has a active condition in HCC --> true
        patient = self.load_patient('hcc001v1')
        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        self.assertTrue(tested.in_denominator())

        # patient has NO active condition in HCC --> true
        patient = self.load_patient('hcc001v1')
        patient.conditions[0]['clinicalStatus'] = 'XXXXXXXX'
        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        self.assertFalse(tested.in_denominator())

        # patient has a condition NOT in HCC --> false
        patient = self.load_patient('hcc001v1')
        patient.conditions[0]['coding'][0]['code'] = 'XXXXXXXX'
        patient.conditions[0]['coding'][1]['code'] = 'XXXXXXXX'
        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        self.assertFalse(tested.in_denominator())

        # patient has NO condition --> false
        patient = self.load_patient('hcc001v1')
        patient.conditions = ConditionRecordSet([])
        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        self.assertFalse(tested.in_denominator())

    def test_in_numerator(self):
        start = arrow.get('2017-08-23 13:24:56')
        end = arrow.get('2018-08-23 13:24:56')
        timeframe = Timeframe(start=start, end=end)

        # patient has a condition in HCC that is assessed within the year --> false
        patient = self.load_patient('hcc001v1')
        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        self.assertFalse(tested.in_numerator())

        # patient has a condition in HCC that is NOT assessed within the year --> true
        patient = self.load_patient('hcc001v1')
        patient.conditions[0]['noteTimestamp'] = '2018-08-22 13:24:56'
        patient.conditions[0]['lastTimestamps']['assessed'] = '2017-08-22 13:24:56'
        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        self.assertTrue(tested.in_numerator())

        # patient has a condition in HCC that is assessed within the year --> false
        patient = self.load_patient('hcc001v1')
        patient.conditions[0]['noteTimestamp'] = '2018-08-22 13:24:56'
        patient.conditions[0]['lastTimestamps']['assessed'] = None
        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        self.assertFalse(tested.in_numerator())

        # patient has a condition in HCC that is NOT assessed within the year --> true
        patient = self.load_patient('hcc001v1')
        patient.conditions[0]['noteTimestamp'] = '2017-08-22 13:24:56'
        patient.conditions[0]['lastTimestamps']['assessed'] = None
        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        self.assertTrue(tested.in_numerator())

        # patient has a condition NOT in HCC that is NOT assessed within the year --> false
        patient = self.load_patient('hcc001v1')
        patient.conditions[0]['lastTimestamps']['assessed'] = '2017-08-22 13:24:56'
        patient.conditions[0]['coding'][0]['code'] = 'XXXXXXXX'
        patient.conditions[0]['coding'][1]['code'] = 'XXXXXXXX'
        tested = Hcc001v1(patient=patient, timeframe=timeframe)
        self.assertFalse(tested.in_numerator())

    def test_in_numerator_with_override(self):
        patient = self.load_patient('hcc001v1')
        patient.protocol_overrides = ProtocolOverrideRecordSet([{
            'protocolKey': 'Hcc001v1',
            'adjustment': {
                'reference': '2018-02-01T00:00:00Z',
                'cycleDays': 180
            },
            'snooze': None,
            'modified': '2018-03-15T12:32:54Z',
        }])  # yapf: disable
        patient.conditions[0]['noteTimestamp'] = '2002-08-22 13:24:56'
        patient.conditions[0]['lastTimestamps']['assessed'] = '2018-08-05 13:24:56'

        tested = Hcc001v1(patient=patient, now=arrow.get('2018-08-05').shift(days=1))
        self.assertFalse(tested.in_numerator())

        tested = Hcc001v1(patient=patient, now=arrow.get('2018-08-05').shift(days=80))
        self.assertFalse(tested.in_numerator())

        tested = Hcc001v1(patient=patient, now=arrow.get('2018-08-05').shift(days=180))
        self.assertFalse(tested.in_numerator())

        tested = Hcc001v1(patient=patient, now=arrow.get('2018-08-05').shift(days=181))
        self.assertTrue(tested.in_numerator())

    def test_compute_results(self):
        start = arrow.get('2017-09-23 13:24:56')
        end = arrow.get('2018-09-23 13:24:56')
        now = arrow.get('2018-09-23 13:24:56')
        timeframe = Timeframe(start=start, end=end)

        # patient has a active condition in HCC assessed within the year --> satisfied
        patient = self.load_patient('hcc001v1')
        tested = Hcc001v1(patient=patient, timeframe=timeframe, now=now)
        result = tested.compute_results()
        self.assertEqual('satisfied', result.status)
        text = 'All Significant Condition have been assessed within the last 12 months.'
        self.assertEqual(text, result.narrative)
        self.assertEqual([], result.recommendations)
        self.assertEqual(333, result.due_in)

        patient.conditions = ConditionRecordSet([
            {
                'coding': [{'system': 'ICD-10', 'code': 'I429', }],
                'clinicalStatus': 'active',
                'periods': [{'from': '2018-05-22', 'to': None}],
                'lastTimestamps': {'assessed': '2018-07-22 13:24:56', },
            },
            {
                'coding': [{'system': 'ICD-10', 'code': 'I429', }],
                'clinicalStatus': 'active',
                'periods': [{'from': '2018-05-22', 'to': None}],
                'lastTimestamps': {'assessed': '2018-06-21 13:24:56', },
            },
            {
                'coding': [{'system': 'ICD-10', 'code': 'I429', }],
                'clinicalStatus': 'active',
                'periods': [{'from': '2018-05-22', 'to': None}],
                'lastTimestamps': {'assessed': '2018-08-22 13:24:56', },
            },
        ])  # yapf: disable
        tested = Hcc001v1(patient=patient, timeframe=timeframe, now=now)
        result = tested.compute_results()
        self.assertEqual(271, result.due_in)

        # patient has a active condition in HCC NOT assessed within the year --> due
        patient = self.load_patient('hcc001v1')
        tested = Hcc001v1(patient=patient, timeframe=timeframe, now=now)
        patient.conditions[0]['lastTimestamps']['assessed'] = '2017-09-22 13:24:56'
        result = tested.compute_results()
        self.assertEqual('due', result.status)
        text = ('Cardiomyopathy, unspecified (I429) is a significant condition '
                'which should be assessed annually.'
                ' The condition was last assessed on 9/22/17 and carries a RAF value of 0.368')
        self.assertEqual(text, result.narrative)
        expected = [
            {
                'title': 'Assess, update or resolve conditions as clinically appropriate',
                'command': {
                    'key': 'assess'
                },
                'key': 'HCC001v1_RECOMMEND_ASSESS_CONDITION',
                'rank': 1,
                'button': 'Assess',
            }, {
                'title': 'Resolve conditions as clinically appropriate',
                'command': {
                    'key': 'resolveCondition'
                },
                'key': 'HCC001v1_RECOMMEND_RESOLVE_CONDITION',
                'rank': 2,
                'button': 'Assess',
            },
        ]  # yapf: disable
        self.helper_exam_recommended(expected, result.recommendations)
        self.assertEqual(-1, result.due_in)

        # patient has NO condition --> false
        patient = self.load_patient('hcc001v1')
        patient.conditions = ConditionRecordSet([])
        tested = Hcc001v1(patient=patient, timeframe=timeframe, now=now)
        result = tested.compute_results()
        self.assertIsNotApplicable(result)

    def test_compute_results_with_override(self):
        patient = self.load_patient('hcc001v1')
        patient.protocol_overrides = ProtocolOverrideRecordSet([{
            'protocolKey': 'Hcc001v1',
            'adjustment': {
                'reference': '2018-02-01T00:00:00Z',
                'cycleDays': 180
            },
            'snooze': None,
            'modified': '2018-03-15T12:32:54Z',
        }])  # yapf: disable
        patient.conditions[0]['noteTimestamp'] = '2002-08-22 13:24:56'
        patient.conditions[0]['lastTimestamps']['assessed'] = '2018-08-05 13:24:56'

        # assessment done within the last 180 days
        tested = Hcc001v1(patient=patient, now=arrow.get('2018-08-05').shift(days=80))
        result = tested.compute_results()
        self.assertEqual('satisfied', result.status)
        text = 'All Significant Condition have been assessed within the last 6 months.'
        self.assertEqual(text, result.narrative)
        self.assertEqual(0, len(result.recommendations))
        self.assertEqual(100, result.due_in)

        # assessment done 181 days ago
        tested = Hcc001v1(patient=patient, now=arrow.get('2018-08-05').shift(days=181))
        result = tested.compute_results()
        self.assertEqual('due', result.status)
        text = ('Cardiomyopathy, unspecified (I429) is a significant condition '
                'which should be assessed annually. The condition was last assessed '
                'on 8/5/18 and carries a RAF value of 0.368')
        self.assertEqual(text, result.narrative)
        self.assertEqual(2, len(result.recommendations))
        self.assertEqual(-1, result.due_in)

    def helper_exam_recommended(self, expected, result):
        for exp, recommendation in zip(expected, result):
            self.assertEqual(exp['key'], recommendation.key)
            self.assertEqual(exp['rank'], recommendation.rank)
            self.assertEqual(exp['button'], recommendation.button)
            self.assertEqual(exp['title'], recommendation.title)
            self.assertEqual(exp['command']['key'], recommendation.command['key'])



from canvas_workflow_kit import settings

from canvas_workflow_kit.tests.base import SDKBaseTest


class SDKBaseTest(SDKBaseTest):

    def setUp(self):
        super().setUp()
        self.mocks_path = f'{settings.BASE_DIR}/tests/sdk/mock_data'



from protocols import events
from protocols.protocol import ClinicalQualityMeasure
from protocols.value_set.v2021 import (
    ClinicalOralEvaluation, DentalCaries, EncounterInpatient, Ethnicity, HospiceCareAmbulatory,
    OncAdministrativeSex, Payer, Race)


class ClinicalQualityMeasure75v9(ClinicalQualityMeasure):
    """
    Children Who Have Dental Decay or Cavities

    Description: Percentage of children, 6 months - 20 years of age, who have had tooth decay or
    cavities during the measurement period

    Definition: None

    Rationale: Dental caries is the most chronic disease among youth aged 6-19 years. Data from the
    National Health and Nutrition Examination Survey from 2015-2016 showed that approximately 45.8%
    of children and youth aged 2-19 years had total caries (untreated and treated). Prevalence of
    total dental caries (untreated and treated) in primary or permanent teeth increases with age,
    going from 21.4%, 50.5%, and 53.8% among ages 2-5, 6-11, and 12-19, respectively. Total dental
    caries was highest in Hispanic youths aged 2-19 at 57.1% compared to 48.1% for non-Hispanic
    black, 44.6% for non-Asian, and 40.4% for non-Hispanic white youth. Monitoring prevalence of
    untreated and total caries is vital to preventing and controlling oral disease (Fleming &
    Afful, 2018).

    Children who have dental decay or cavities are less likely to be in very good or excellent
    overall health than children without decay or cavities (Edelstein & Chinn, 2009). Children with
    decay are also more likely to have other oral health problems such as toothaches, broken teeth,
    and bleeding gums (Data Resource Center for Child and Adolescent Health, 2007).

    Guidance: This eCQM is a patient-based measure.

    This version of the eCQM uses QDM version 5.5. Please refer to the eCQI resource center
    (https://ecqi.healthit.gov/qdm) for more information on the QDM.

    More information: https://ecqi.healthit.gov/ecqm/measures/cms75v9
    """

    title = 'Children Who Have Dental Decay or Cavities'

    identifiers = ['CMS75v9']

    types = ['CQM']

    responds_to_event_types = [
        events.HEALTH_MAINTENANCE,
    ]

    authors = [
        'National Committee for Quality Assurance',
    ]

    references = [
        'Data Resource Center for Child and Adolescent Health, Child and Adolescent Health Measurement Initiative. (2007). 2007 National Survey of Children’s Health. Baltimore, MD: Author.',
        'Edelstein, B. L., & Chinn, C. H. (2009). Update on disparities in oral health and access to dental care for America’s children. Academic Pediatrics, 9(6), 415-419.',
        'Fleming, E., & Afful, J. (2018). Prevalence of total and untreated dental carries among youth: United States, 2015-2016. NCHS Data Brief No. 307. Hyattsville, MD: National Center for Health Statistics.',
    ]

    funding_source = ''

    def in_initial_population(self):
        """
        Initial population: Children, 6 months - 20 years of age, with a clinical oral evaluation
        during the measurement period
        """
        pass

    def in_denominator(self):
        """
        Denominator: Equals Initial Population

        Exclusions: Exclude patients whose hospice care overlaps the measurement period

        Exceptions: None
        """
        return self.in_initial_population()

    def in_numerator(self):
        """
        Numerator: Children who had a diagnosis of cavities or decayed teeth overlapping the
        measurement period

        Exclusions: Not applicable
        """
        pass


    def compute_results(self):
        """
        Clinical recommendation: This is an outcome measure. As such, no clinical recommendations
        are included.
        """
        pass



import arrow

from canvas_workflow_kit.timeframe import Timeframe
from canvas_workflow_kit.value_set.v2017 import LdlC, ProthrombinTime

from .base import SDKBaseTest


class SDKLabTest(SDKBaseTest):

    def test_lab_reports(self):
        patient = self.load_patient('example')
        self.assertTrue(patient)
        self.assertEqual(patient.first_name, 'Darey')

        # look for labs he doesn't have
        t = Timeframe(arrow.get('2017-01-17'), arrow.get('2018-01-17'))
        self.assertFalse(patient.lab_reports.find(ProthrombinTime).within(t).last())

        # look for a lab he does have
        self.assertTrue(patient.lab_reports.find(LdlC).within(t).last())

        # and the value should come back
        self.assertEqual(patient.lab_reports.find(LdlC).within(t).last()['value'], '120')

        # but if we move the timeframe it shouldn't come back
        t2 = Timeframe(arrow.get('2016-01-17'), arrow.get('2017-01-17'))
        self.assertFalse(patient.lab_reports.find(LdlC).within(t2).last())
        self.assertEqual(patient.lab_reports.find(LdlC).within(t2).last(), None)



from ..value_set import ValueSet


class AntiHepatitisAIggAntigenTest(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent laboratory tests to detect anti hepatitis A immunoglobulin (IgG) antibodies.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Laboratory Test

    **Inclusion Criteria:** Includes only relevant concepts associated with test codes for IgG antibody to hepatitis A.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.198.12.1033'
    VALUE_SET_NAME = 'Anti Hepatitis A IgG Antigen Test'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '32018-4',
        '40724-7',
        '51913-2'
    }

class AntiHepatitisBVirusSurfaceAb(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent laboratory tests to detect anti hepatitis B surface antigen in the blood.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Laboratory Test, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with test codes for anti-surface antibody in the blood related to hepatitis B virus.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.198.12.1073'
    VALUE_SET_NAME = 'Anti Hepatitis B Virus Surface Ab'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '10900-9',
        '22322-2',
        '39535-0',
        '48070-7',
        '49177-9',
        '75409-3'
    }

class ChlamydiaScreening(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent laboratory testing for chlamydia infections.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Laboratory Test, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with tests for chlamydia infection. This is a grouping of LOINC codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.110.12.1052'
    VALUE_SET_NAME = 'Chlamydia Screening'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '14463-4',
        '14464-2',
        '14467-5',
        '14474-1',
        '14513-6',
        '16600-9',
        '21190-4',
        '21191-2',
        '21613-5',
        '23838-6',
        '31775-0',
        '31777-6',
        '36902-5',
        '36903-3',
        '42931-6',
        '43304-5',
        '43404-3',
        '43405-0',
        '43406-8',
        '44806-8',
        '44807-6',
        '45068-4',
        '45069-2',
        '45075-9',
        '45076-7',
        '45084-1',
        '45091-6',
        '45095-7',
        '45098-1',
        '45100-5',
        '47211-8',
        '47212-6',
        '49096-1',
        '4993-2',
        '50387-0',
        '53925-4',
        '53926-2',
        '557-9',
        '560-3',
        '6349-5',
        '6354-5',
        '6355-2',
        '6356-0',
        '6357-8',
        '80360-1',
        '80361-9',
        '80362-7'
    }

class FecalOccultBloodTestFobt(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent a fecal occult blood test.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Laboratory Test, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying tests for occult blood in stool. This is a grouping of LOINC codes.

    **Exclusion Criteria:** Excludes tests for occult blood in other parts of the body. Excludes codes that indicate that this test was ordered only, and not necessarily performed.
    """

    OID = '2.16.840.1.113883.3.464.1003.198.12.1011'
    VALUE_SET_NAME = 'Fecal Occult Blood Test (FOBT)'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '12503-9',
        '12504-7',
        '14563-1',
        '14564-9',
        '14565-6',
        '2335-8',
        '27396-1',
        '27401-9',
        '27925-7',
        '27926-5',
        '29771-3',
        '56490-6',
        '56491-4',
        '57905-2',
        '58453-2',
        '80372-6'
    }

class FitDna(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent a fecal immunochemical (FIT) DNA test.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Laboratory Test, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying a FIT DNA laboratory test. This is a grouping of LOINC codes.

    **Exclusion Criteria:** Excludes codes that indicate that this test was ordered only, and not necessarily performed.
    """

    OID = '2.16.840.1.113883.3.464.1003.108.12.1039'
    VALUE_SET_NAME = 'FIT DNA'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '77353-1',
        '77354-9'
    }

class GroupAStreptococcusTest(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent laboratory tests for group A streptococcus.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Laboratory Test.

    **Inclusion Criteria:** Includes only relevant concepts associated with laboratory tests used to identify the presence of group A streptococcus in the throat.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.198.12.1012'
    VALUE_SET_NAME = 'Group A Streptococcus Test'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '11268-0',
        '17656-0',
        '17898-8',
        '18481-2',
        '31971-5',
        '49610-9',
        '5036-9',
        '60489-2',
        '626-2',
        '6557-3',
        '6558-1',
        '6559-9',
        '68954-7',
        '78012-2'
    }

class Hba1CLaboratoryTest(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent hemoglobin A1c laboratory tests.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Laboratory Test.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying hemoglobin A1c laboratory tests.

    **Exclusion Criteria:** Excludes codes that identify hemoglobin A1c laboratory tests that use the International Federation of Clinical Chemistry and Laboratory Medicine (IFCC)   protocol and Japanese Diabetes Society (JDS)/Japanese Society of Clinical Chemistry (JSCC) protocol.
    """

    OID = '2.16.840.1.113883.3.464.1003.198.12.1013'
    VALUE_SET_NAME = 'HbA1c Laboratory Test'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '17856-6',
        '4548-4',
        '4549-2'
    }

class HpvTest(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent identify high-risk human papilloma virus (HPV) tests.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Laboratory Test, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with high-risk HPV tests performed on cervical samples. This is a grouping of LOINC codes.

    **Exclusion Criteria:** Excludes high-risk HPV tests conducted on non-cervical samples. Excludes codes that indicate that this test was ordered only, and not necessarily performed.
    """

    OID = '2.16.840.1.113883.3.464.1003.110.12.1059'
    VALUE_SET_NAME = 'HPV Test'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '21440-3',
        '30167-1',
        '38372-9',
        '59263-4',
        '59264-2',
        '59420-0',
        '69002-4',
        '71431-1',
        '75406-9',
        '75694-0',
        '77379-6',
        '77399-4',
        '77400-0',
        '82354-2',
        '82456-5',
        '82675-0'
    }

class HumanImmunodeficiencyVirusHivLaboratoryTestCodesAbAndAg(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent laboratory tests used for Human Immunodeficiency Virus screening.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Laboratory Test. Used for HIV screening.

    **Inclusion Criteria:** Includes only relevant concepts associated with laboratory tests for HIV-1 and HIV-2 antibodies or antigens.

    **Exclusion Criteria:** Excludes tests and procedures that might be associated with HIV infection that are not used for screening or testing to establish an HIV diagnosis, which would include home/self HIV testing, HIV genotyping tests, HIV RNA tests, HIV cultures, clinical codes used to document care provided to HIV-infected patients.
    """

    OID = '2.16.840.1.113762.1.4.1056.50'
    VALUE_SET_NAME = 'Human Immunodeficiency Virus (HIV) Laboratory Test Codes (Ab and Ag)'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '10901-7',
        '10902-5',
        '11078-3',
        '11079-1',
        '11080-9',
        '11081-7',
        '11082-5',
        '12855-3',
        '12856-1',
        '12857-9',
        '12858-7',
        '12859-5',
        '12870-2',
        '12871-0',
        '12872-8',
        '12875-1',
        '12876-9',
        '12893-4',
        '12894-2',
        '12895-9',
        '13499-9',
        '13920-4',
        '14092-1',
        '14126-7',
        '16132-3',
        '16974-8',
        '16975-5',
        '16976-3',
        '16977-1',
        '16978-9',
        '16979-7',
        '18396-2',
        '19110-6',
        '21007-0',
        '21331-4',
        '21332-2',
        '21334-8',
        '21335-5',
        '21336-3',
        '21337-1',
        '21338-9',
        '21339-7',
        '21340-5',
        '22356-0',
        '22357-8',
        '22358-6',
        '24012-7',
        '28004-0',
        '28052-9',
        '29327-4',
        '29893-5',
        '30361-0',
        '31072-2',
        '31073-0',
        '31201-7',
        '31430-2',
        '32571-2',
        '32602-5',
        '32827-8',
        '32842-7',
        '33508-3',
        '33660-2',
        '33806-1',
        '33807-9',
        '33866-5',
        '34591-8',
        '34592-6',
        '35437-3',
        '35438-1',
        '35439-9',
        '35440-7',
        '35441-5',
        '35442-3',
        '35443-1',
        '35444-9',
        '35445-6',
        '35446-4',
        '35447-2',
        '35448-0',
        '35449-8',
        '35450-6',
        '35452-2',
        '35564-4',
        '35565-1',
        '40437-6',
        '40438-4',
        '40439-2',
        '40732-0',
        '40733-8',
        '41143-9',
        '41144-7',
        '41145-4',
        '41290-8',
        '42339-2',
        '42600-7',
        '42627-0',
        '42768-2',
        '43008-2',
        '43009-0',
        '43010-8',
        '43011-6',
        '43012-4',
        '43013-2',
        '43185-8',
        '43599-0',
        '44531-2',
        '44532-0',
        '44533-8',
        '44607-0',
        '44872-0',
        '44873-8',
        '45212-8',
        '47029-4',
        '48345-3',
        '48346-1',
        '49483-1',
        '49580-4',
        '49718-0',
        '49905-3',
        '49965-7',
        '51786-2',
        '51866-2',
        '5220-9',
        '5221-7',
        '5222-5',
        '5223-3',
        '5224-1',
        '5225-8',
        '53379-4',
        '53601-1',
        '54086-4',
        '56888-1',
        '57974-8',
        '57975-5',
        '57976-3',
        '57977-1',
        '57978-9',
        '58900-2',
        '62456-9',
        '68961-2',
        '69668-2',
        '73905-2',
        '73906-0',
        '75622-1',
        '75666-8',
        '77685-6',
        '7917-8',
        '7918-6',
        '7919-4',
        '80203-3',
        '80387-4',
        '81641-3',
        '83101-6',
        '85037-0',
        '85686-4',
        '86233-4',
        '86657-4',
        '89365-1',
        '89374-3',
        '9660-2',
        '9661-0',
        '9662-8',
        '9663-6',
        '9664-4',
        '9665-1',
        '9666-9',
        '9667-7',
        '9668-5',
        '9669-3',
        '9821-0'
    }

class LabTestsDuringPregnancy(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent lab tests conducted during pregnancy (e.g., tests conducted on amniotic fluid).

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Laboratory Test.

    **Inclusion Criteria:** Includes only relevant concepts associated with laboratory tests conducted during pregnancy. This is a grouping of LOINC codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.111.12.1007'
    VALUE_SET_NAME = 'Lab Tests During Pregnancy'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '15019-3',
        '1832-5',
        '1834-1',
        '19171-8',
        '19176-7',
        '19177-5',
        '20403-2',
        '20404-0',
        '31993-9',
        '33773-3',
        '34493-7',
        '34656-9',
        '34718-7',
        '35457-1',
        '41273-4',
        '41274-2',
        '42316-0',
        '43798-8',
        '45273-0',
        '45331-6',
        '45332-4',
        '46731-6',
        '46989-0',
        '48030-1',
        '48039-2',
        '48781-9',
        '49246-2',
        '49318-9',
        '64088-8',
        '64094-6'
    }

class LabTestsForSexuallyTransmittedInfections(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent laboratory tests for sexually transmitted infections.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Laboratory Test, Order.

    **Inclusion Criteria:** Includes only relevant concepts associated with laboratory tests for sexually transmitted infections in females. This is a grouping of LOINC codes.

    **Exclusion Criteria:** Excludes codes that indicate lab tests for sexually transmitted infections in male patients.
    """

    OID = '2.16.840.1.113883.3.464.1003.110.12.1051'
    VALUE_SET_NAME = 'Lab Tests for Sexually Transmitted Infections'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '10705-2',
        '11083-3',
        '11084-1',
        '11481-9',
        '11597-2',
        '12222-6',
        '12223-4',
        '14499-8',
        '14500-3',
        '14502-9',
        '14503-7',
        '14504-5',
        '14506-0',
        '16280-0',
        '17398-9',
        '17399-7',
        '17400-3',
        '17401-1',
        '17402-9',
        '17403-7',
        '17404-5',
        '17405-2',
        '17406-0',
        '17407-8',
        '17408-6',
        '17409-4',
        '17410-2',
        '17411-0',
        '17412-8',
        '17723-8',
        '17724-6',
        '17725-3',
        '17726-1',
        '17727-9',
        '17728-7',
        '17729-5',
        '20507-0',
        '20508-8',
        '21414-8',
        '21415-5',
        '21416-3',
        '21440-3',
        '21441-1',
        '22461-8',
        '22462-6',
        '22587-0',
        '22590-4',
        '22592-0',
        '22594-6',
        '24110-9',
        '24111-7',
        '24312-1',
        '26009-1',
        '29311-8',
        '30167-1',
        '31147-2',
        '31905-3',
        '31906-1',
        '32198-4',
        '32199-2',
        '32705-6',
        '34147-9',
        '34382-2',
        '38372-9',
        '40679-3',
        '40680-1',
        '42481-2',
        '43305-2',
        '43403-5',
        '44543-7',
        '44544-5',
        '44546-0',
        '44547-8',
        '44549-4',
        '44550-2',
        '47236-5',
        '47237-3',
        '47238-1',
        '47387-6',
        '48560-7',
        '49891-5',
        '49896-4',
        '5028-6',
        '50388-8',
        '50690-7',
        '51838-1',
        '51839-9',
        '5291-0',
        '5292-8',
        '53605-2',
        '53762-1',
        '53879-3',
        '5392-6',
        '53927-0',
        '5393-4',
        '5394-2',
        '55299-2',
        '57032-5',
        '59263-4',
        '59264-2',
        '59420-0',
        '61372-9',
        '61373-7',
        '61374-5',
        '61375-2',
        '61376-0',
        '61377-8',
        '61378-6',
        '61379-4',
        '61380-2',
        '61381-0',
        '61382-8',
        '61383-6',
        '61384-4',
        '61385-1',
        '61386-9',
        '61387-7',
        '61388-5',
        '61389-3',
        '61390-1',
        '61391-9',
        '61392-7',
        '61393-5',
        '61394-3',
        '61395-0',
        '61396-8',
        '63464-2',
        '6487-3',
        '6488-1',
        '6489-9',
        '6510-2',
        '6511-0',
        '6514-4',
        '6516-9',
        '6561-5',
        '6562-3',
        '660-1',
        '688-2',
        '690-8',
        '69002-4',
        '691-6',
        '692-4',
        '693-2',
        '698-1',
        '71431-1',
        '71793-4',
        '73732-0',
        '73752-8',
        '73959-9',
        '7975-6',
        '8041-6'
    }

class LdlCholesterol(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent lab tests commonly used for low-density lipoproteins (LDL) cholesterol measurement.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) category related to Laboratory Test. The intent of this data element is to identify patients who had a lab test on a source of serum or plasma.

    **Inclusion Criteria:** Includes only relevant concepts associated with LDL-C tests using the source of serum or plasma based on a measurement scale of mass per volume.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.526.3.1573'
    VALUE_SET_NAME = 'LDL Cholesterol'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '13457-7',
        '18261-8',
        '18262-6',
        '2089-1',
        '43394-6',
        '49132-4',
        '50193-2',
        '55440-2',
        '86911-5',
        '90364-1',
        '91105-7',
        '91106-5',
        '91107-3',
        '91108-1',
        '91109-9',
        '91110-7',
        '91111-5'
    }

class MeaslesAntibodyTestIggAntibodyPresence(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent test results related to IgG antibodies and measles. A positive test conveys immunologic protection against measles.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Laboratory Test, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying positive antibody tests for measles IgG In the serum, cerebrospinal fluid or body fluid.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.198.12.1060'
    VALUE_SET_NAME = 'Measles Antibody Test (IgG Antibody presence)'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '20479-2',
        '35275-7',
        '40648-8',
        '41132-2',
        '53536-9'
    }

class MeaslesAntibodyTestIggAntibodyTiter(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent laboratory tests for the  titre (or amount) of IgG antibodies to measles.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Laboratory Test, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with test codes for measles IgG antibody titres in the serum, or cerebrospinal fluid.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.198.12.1059'
    VALUE_SET_NAME = 'Measles Antibody Test (IgG Antibody Titer)'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '21500-4',
        '21501-2',
        '22501-1',
        '22502-9'
    }

class MumpsAntibodyTestIggAntibodyPresence(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent test results for presence of mumps IgG antibody.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Laboratory Test, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying positive antibody tests for mumps IgG.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.198.12.1062'
    VALUE_SET_NAME = 'Mumps Antibody Test (IgG Antibody presence)'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '22415-4',
        '39011-2',
        '40737-9',
        '6476-6',
        '74422-7'
    }

class MumpsAntibodyTestIggAntibodyTiter(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent test results for titre (amount) of mumps IgG antibody.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Laboratory Test, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying titres for mumps antibody IgG in serum and by immunofluorescence in cerebrospinal fluid or serum.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.198.12.1061'
    VALUE_SET_NAME = 'Mumps Antibody Test (IgG Antibody Titer)'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '21401-5',
        '22416-2',
        '22417-0',
        '6477-4'
    }

class PapTest(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent identify cervical cytology tests.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Laboratory Test, Order or Laboratory Test, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with cervical cytology tests. This is a grouping of LOINC codes.

    **Exclusion Criteria:** Excludes cytology performed on non-cervical samples. Excludes codes that indicate that this test was ordered only, and not necessarily performed.
    """

    OID = '2.16.840.1.113883.3.464.1003.108.12.1017'
    VALUE_SET_NAME = 'Pap Test'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '10524-7',
        '18500-9',
        '19762-4',
        '19764-0',
        '19765-7',
        '19766-5',
        '19774-9',
        '33717-0',
        '47527-7',
        '47528-5'
    }

class PregnancyTest(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent urine, serum, or plasma pregnancy tests.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Laboratory Test, Order.

    **Inclusion Criteria:** Includes only relevant concepts associated with conducting a pregnancy test. This is a grouping of LOINC codes.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.111.12.1011'
    VALUE_SET_NAME = 'Pregnancy Test'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '19080-1',
        '19180-9',
        '20415-6',
        '20994-0',
        '2106-3',
        '2107-1',
        '2110-5',
        '2111-3',
        '2112-1',
        '2113-9',
        '2114-7',
        '2115-4',
        '2118-8',
        '2119-6',
        '21198-7',
        '25372-4',
        '25373-2',
        '34670-0',
        '45194-8',
        '55869-2',
        '55870-0',
        '56497-1',
        '80384-1',
        '83086-9'
    }

class ProstateSpecificAntigenTest(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent prostate specific antigen (PSA) test.

    **Data Element Scope:** This value set may use Quality Data Model (QDM) category related to Laboratory Test.

    **Inclusion Criteria:** Includes only relevant concepts associated with prostate specific antigen (PSA) tests.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.526.3.401'
    VALUE_SET_NAME = 'Prostate Specific Antigen Test'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '10508-0',
        '10886-0',
        '12841-3',
        '15323-9',
        '15324-7',
        '15325-4',
        '19195-7',
        '19201-3',
        '2857-1',
        '33667-7',
        '34611-4',
        '35741-8'
    }

class RubellaAntibodyTestIggAntibodyPresence(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent laboratory tests for the presence of rubella IgG antibody in serum.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Laboratory Test, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with testing the presence of rubella IgG antibody.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.198.12.1064'
    VALUE_SET_NAME = 'Rubella Antibody Test (IgG Antibody presence)'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '25514-1',
        '39013-8',
        '40667-8',
        '40668-6',
        '63462-6',
        '74415-1'
    }

class RubellaAntibodyTestIggAntibodyTiter(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent laboratory tests for the titre (amount) of rubella IgG antibody in serum.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Laboratory Test, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with testing to identify titres for rubella IgG antibody.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.198.12.1063'
    VALUE_SET_NAME = 'Rubella Antibody Test (IgG Antibody Titer)'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '41763-4',
        '46110-3'
    }

class UrineProteinTests(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent urine protein tests.

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Laboratory Test, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with identifying urine protein tests, including those that look for albumin, microalbumin, and protein.

    **Exclusion Criteria:** Excludes codes that identify urine protein tests that would be taken from a fetus.
    """

    OID = '2.16.840.1.113883.3.464.1003.109.12.1024'
    VALUE_SET_NAME = 'Urine Protein Tests'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '11218-5',
        '12842-1',
        '13705-9',
        '13801-6',
        '13986-5',
        '13992-3',
        '14956-7',
        '14957-5',
        '14958-3',
        '14959-1',
        '1753-3',
        '1754-1',
        '1755-8',
        '1757-4',
        '17819-4',
        '18373-1',
        '20454-5',
        '20621-9',
        '21059-1',
        '21482-5',
        '26801-1',
        '27298-9',
        '2887-8',
        '2888-6',
        '2889-4',
        '2890-2',
        '29946-1',
        '30000-4',
        '30001-2',
        '30003-8',
        '32209-9',
        '32294-1',
        '32551-4',
        '34366-5',
        '35663-4',
        '40486-3',
        '40662-9',
        '40663-7',
        '43605-5',
        '43606-3',
        '43607-1',
        '44292-1',
        '47558-2',
        '49002-9',
        '49023-5',
        '50209-6',
        '50561-0',
        '50949-7',
        '51190-7',
        '53121-0',
        '53525-2',
        '53530-2',
        '53531-0',
        '53532-8',
        '56553-1',
        '57369-1',
        '57735-3',
        '5804-0',
        '58448-2',
        '58992-9',
        '59159-4',
        '60678-0',
        '63474-1',
        '6941-9',
        '6942-7',
        '76401-9',
        '77253-3',
        '77254-1',
        '77940-5',
        '9318-7'
    }

class VaricellaZosterAntibodyTestIggAntibodyPresence(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent laboratory tests measuring the presence of IgG antibody to varicella zoster in serum, cerebrospinal fluid (CSF

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Laboratory Test, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with tests measuring the presence of IgG antibody to varicella zoster in serum, CSF and body fluids.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.198.12.1067'
    VALUE_SET_NAME = 'Varicella Zoster Antibody Test (IgG Antibody Presence)'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '15410-4',
        '19162-7',
        '41512-5',
        '42537-1',
        '53534-4'
    }

class VaricellaZosterAntibodyTestIggAntibodyTiter(ValueSet):
    """
    **Clinical Focus:** This value set contains concepts that represent laboratory tests measuring the titre of IgG antibody to varicella zoster in serum and cerebrospinal fluid (CSF).

    **Data Element Scope:** This value set may use the Quality Data Model (QDM) datatype related to Laboratory Test, Performed.

    **Inclusion Criteria:** Includes only relevant concepts associated with tests measuring the titre or amount of IgG antibody to varicella zoster in serum or CSF.

    **Exclusion Criteria:** No exclusions.
    """

    OID = '2.16.840.1.113883.3.464.1003.198.12.1066'
    VALUE_SET_NAME = 'Varicella Zoster Antibody Test (IgG Antibody Titer)'
    EXPANSION_VERSION = 'eCQM Update 2020-05-07'

    LOINC = {
        '21595-4',
        '22601-9',
        '22602-7',
        '6569-8'
    }


import arrow

from canvas_workflow_kit import settings

from canvas_workflow_kit.builtin_cqms.hcc005v1_annual_wellness_visit import Hcc005v1
from canvas_workflow_kit.patient_recordset import BillingLineItemRecordSet, ProtocolOverrideRecordSet
from canvas_workflow_kit.tests.base import SDKBaseTest
from canvas_workflow_kit.timeframe import Timeframe


class TestHcc005v1(SDKBaseTest):

    def setUp(self):
        self.mocks_path = f'{settings.BASE_DIR}/builtin_cqms/tests/mock_data'

    def test_enabled(self):
        self.assertTrue(Hcc005v1.enabled())

    def test_description(self):
        expected = 'Patient 65 or older due  for Annual Wellness Visit.'
        self.assertEqual(expected, Hcc005v1._meta.description)

    def test_information(self):
        expected = ('https://canvas-medical.zendesk.com/hc/en-us/articles/360059083973-Annual-Wellness-Visit-HCC005v1')
        self.assertEqual(expected, Hcc005v1._meta.information)

    def test_change_types(self):
        result = Hcc005v1._meta.compute_on_change_types
        expected = [
            'protocol_override',
            'billing_line_item',
            'patient',
        ]
        self.assertEqual(len(expected), len(result))
        for change in expected:
            self.assertIn(change, result)

    def test_in_initial_population(self):
        start = arrow.get('2017-08-23 13:24:56')
        end = arrow.get('2018-08-23 13:24:56')
        timeframe = Timeframe(start=start, end=end)
        patient = self.load_patient('hcc005v1')
        tested = Hcc005v1(patient=patient, timeframe=timeframe)
        self.assertTrue(tested.in_initial_population())

        start = arrow.get('2011-08-23 13:24:56')
        end = arrow.get('2012-08-23 13:24:56')
        timeframe = Timeframe(start=start, end=end)
        patient = self.load_patient('hcc005v1')
        tested = Hcc005v1(patient=patient, timeframe=timeframe)
        self.assertFalse(tested.in_initial_population())

    def test_in_denominator(self):
        start = arrow.get('2017-08-23 13:24:56')
        end = arrow.get('2018-08-23 13:24:56')
        timeframe = Timeframe(start=start, end=end)
        patient = self.load_patient('hcc005v1')
        tested = Hcc005v1(patient=patient, timeframe=timeframe)
        self.assertTrue(tested.in_denominator())

        start = arrow.get('2011-08-23 13:24:56')
        end = arrow.get('2012-08-23 13:24:56')
        timeframe = Timeframe(start=start, end=end)
        patient = self.load_patient('hcc005v1')
        tested = Hcc005v1(patient=patient, timeframe=timeframe)
        self.assertFalse(tested.in_denominator())

    def test_in_numerator_patient_with_wellness_cpt(self):
        start = arrow.get('2017-08-23 13:24:56')
        end = arrow.get('2018-08-23 13:24:56')
        timeframe = Timeframe(start=start, end=end)

        tests = [
            ('G0437', True),
            ('G0438', False),
            ('G0439', False),
            ('G0402', False),
            ('99387', False),
            ('99397', False),
            ('99999', True),
        ]
        for code, expected in tests:
            patient = self.load_patient('hcc005v1')
            patient.billing_line_items = BillingLineItemRecordSet([{
                'id': 92,
                'created': '2018-07-13T21:41:21.407046Z',
                'datetimeOfService': '2018-07-13T21:41:21.407046Z',
                'cpt': code,
                'units': 1
            }])
            tested = Hcc005v1(patient=patient, timeframe=timeframe)
            if expected:
                self.assertTrue(tested.in_numerator())
            else:
                self.assertFalse(tested.in_numerator())

            patient = self.load_patient('hcc005v1')
            patient.billing_line_items = BillingLineItemRecordSet([{
                'id': 92,
                'created': '2017-07-13T21:41:21.407046Z',
                'datetimeOfService': '2017-07-13T21:41:21.407046Z',
                'cpt': code,
                'units': 1
            }])
            tested = Hcc005v1(patient=patient, timeframe=timeframe)
            self.assertTrue(tested.in_numerator())

    def test_in_numerator_patient_without_wellness_cpt(self):
        start = arrow.get('2017-08-23 13:24:56')
        end = arrow.get('2018-08-23 13:24:56')

        timeframe = Timeframe(start=start, end=end)
        patient = self.load_patient('hcc005v1')
        tested = Hcc005v1(patient=patient, timeframe=timeframe)
        self.assertTrue(tested.in_numerator())

    def test_compute_results_patient_with_wellness_cpt(self):
        start = arrow.get('2017-08-23 13:24:56')
        end = arrow.get('2018-08-23 13:24:56')
        now = arrow.get('2018-12-15 13:24:56')
        timeframe = Timeframe(start=start, end=end)
        patient = self.load_patient('hcc005v1')
        patient.billing_line_items = BillingLineItemRecordSet([{
            'id': 92,
            'created': '2018-07-13T21:41:21.407046Z',
            'datetimeOfService': '2018-07-13T21:41:21.407046Z',
            'cpt': 'G0438',
            'units': 1
        }])
        tested = Hcc005v1(patient=patient, timeframe=timeframe, now=now)
        result = tested.compute_results()
        self.assertEqual('satisfied', result.status)
        text = r'^Jim had a visit [\w\s]+ on 7/13/18.$'
        self.assertRegex(result.narrative, text)
        self.assertEqual([], result.recommendations)
        self.assertEqual(210, result.due_in)

    def test_compute_results_patient_without_wellness_cpt(self):
        start = arrow.get('2017-08-23 13:24:56')
        end = arrow.get('2018-08-23 13:24:56')
        timeframe = Timeframe(start=start, end=end)
        patient = self.load_patient('hcc005v1')
        tested = Hcc005v1(patient=patient, timeframe=timeframe)
        result = tested.compute_results()
        self.assertEqual('due', result.status)
        text = ('Jim is due for a Annual Wellness Visit.\n'
                'There are no Annual Wellness Visits on record.')
        self.assertEqual(text, result.narrative)
        expected = [
            {
                'title': 'Schedule for Annual Wellness Visit',
                'command': {
                    'type': 'instruct',
                    'filter': {
                        'coding': [{'system': 'hcpcs', 'code': [
                            '99387',
                            '99397',
                            'G0402',
                            'G0438',
                            'G0439',
                        ], }]}},
                'key': 'HCC005v1_RECOMMEND_WELLNESS_VISIT',
                'rank': 1,
                'button': 'Plan',
            },
        ]  # yapf: disable
        self.helper_exam_recommended(expected, result.recommendations)
        self.assertEqual(-1, result.due_in)

    def test_compute_results_with_override(self):
        patient = self.load_patient('hcc005v1')
        patient.protocol_overrides = ProtocolOverrideRecordSet([{
            'protocolKey': 'Hcc005v1',
            'adjustment': {
                'reference': '2018-02-01T00:00:00Z',
                'cycleDays': 180
            },
            'snooze': None,
            'modified': '2018-03-15T12:32:54Z',
        }])  # yapf: disable
        patient.billing_line_items = BillingLineItemRecordSet([{
            'id': 92,
            'created': '2018-08-05T21:41:21.407046Z',
            'datetimeOfService': '2018-08-05T21:41:21.407046Z',
            'cpt': 'G0438',
            'units': 1
        }])
        # exam done within the last 180 days
        tested = Hcc005v1(patient=patient, now=arrow.get('2018-08-05').shift(days=80))
        result = tested.compute_results()
        self.assertEqual('satisfied', result.status)
        text = r'^Jim had a visit [\w\s]+ on 8/5/18.$'
        self.assertRegex(result.narrative, text)
        self.assertEqual(0, len(result.recommendations))
        self.assertEqual(100, result.due_in)

        # exam done 181 days
        tested = Hcc005v1(patient=patient, now=arrow.get('2018-08-05').shift(days=181))
        result = tested.compute_results()
        self.assertEqual('due', result.status)
        text = ('Jim is due for a Annual Wellness Visit.\n'
                'Last Annual Wellness Visit was 6 months ago on 8/5/18.')
        self.assertEqual(text, result.narrative)
        self.assertEqual(1, len(result.recommendations))
        self.assertEqual(-1, result.due_in)

    def helper_exam_recommended(self, expected, result):
        for exp, recommendation in zip(expected, result):
            self.assertEqual(exp['key'], recommendation.key)
            self.assertEqual(exp['rank'], recommendation.rank)
            self.assertEqual(exp['button'], recommendation.button)
            self.assertEqual(exp['title'], recommendation.title)
            self.assertEqual(exp['command']['type'], recommendation.command['type'])
            exp_item = exp['command']['filter']['coding']
            rec_item = recommendation.command['filter']['coding']
            self.assertEqual(len(exp_item), len(rec_item))
            for exp_coding, rec_coding in zip(exp_item, rec_item):
                self.assertEqual(exp_coding['system'], rec_coding['system'])
                self.assertEqual(sorted(exp_coding['code']), sorted(rec_coding['code']))



from protocols import events
from protocols.protocol import ClinicalQualityMeasure
from protocols.value_set.v2021 import (
    ClinicalOralEvaluation, EncounterInpatient, Ethnicity, HospiceCareAmbulatory, OfficeVisit,
    OncAdministrativeSex, Payer, PreventiveCareEstablishedOfficeVisit0To17,
    PreventiveCareServicesEstablishedOfficeVisit18AndUp,
    PreventiveCareServicesInitialOfficeVisit0To17, PreventiveCareServicesInitialOfficeVisit18AndUp,
    Race)


class ClinicalQualityMeasure74v10(ClinicalQualityMeasure):
    """
    Primary Caries Prevention Intervention as Offered by Primary Care Providers, including Dentists

    Description: Percentage of children, 6 months - 20 years of age, who received a fluoride
    varnish application during the measurement period

    Definition: None

    Rationale: The literature reflects that fluoride varnish, when applied to the teeth of high-
    risk children, reduces, in conjunction with anticipatory guidance provided to the caregiver,
    the risk of the child developing caries (Weintraub et al., 2006).

    Guidance: This eCQM is a patient-based measure.

    This version of the eCQM uses QDM version 5.5. Please refer to the eCQI resource center
    (https://ecqi.healthit.gov/qdm) for more information on the QDM.

    More information: https://ecqi.healthit.gov/ecqm/measures/cms74v10
    """

    title = 'Primary Caries Prevention Intervention as Offered by Primary Care Providers, including Dentists'

    identifiers = ['CMS74v10']

    types = ['CQM']

    responds_to_event_types = [
        events.HEALTH_MAINTENANCE,
    ]

    authors = [
        'National Committee for Quality Assurance',
    ]

    references = [
        'Weintraub, J. A., Ramos-Gomez, F., Jue, B., et al. (2006). Fluoride varnish efficacy in preventing early childhood caries. Journal of Dental Research, 85(2), 172-176.',
        'Weyant, R. J., Tracy, S. L., Anselmo, T. T., et al. (2013). Topical fluoride for caries prevention: Executive summary of the updated clinical recommendations and supporting systematic review. Journal of the American Dental Association, 144(11), 1279-1291. doi: 10.14219/jada.archive.2013.0057',
        'Moyer, V.A. on behalf of the US Preventive Services Task Force. (2014). Prevention of dental caries in children from birth through age 5 years: US Preventive Services Task Force recommendation statement. Pediatrics, 133(6), 1102-1111.',
    ]

    funding_source = ''

    def in_initial_population(self):
        """
        Initial population: Children, 6 months - 20 years of age, with a visit during the
        measurement period
        """
        pass

    def in_denominator(self):
        """
        Denominator: Equals Initial Population

        Exclusions: Exclude patients whose hospice care overlaps the measurement period

        Exceptions: None
        """
        return self.in_initial_population()

    def in_numerator(self):
        """
        Numerator: Children who receive a fluoride varnish application during the measurement
        period

        Exclusions: Not Applicable
        """
        pass


    def compute_results(self):
        """
        Clinical recommendation: ADA clinical recommendations for use of professionally applied or
        prescription-strength, home-use topical fluorides for caries prevention in patients at
        elevated risk of developing caries:

        - Younger Than 6 Years - 2.26 percent fluoride varnish at least every three to six months

        - 6-18 Years - 2.26 percent fluoride varnish at least every three to six months OR 1.23
        percent fluoride (APF*) gel for four minutes at least every three to six months

        - Older Than 18 Years - 2.26 percent fluoride varnish at least every three to six months OR
        1.23 percent fluoride (APF) gel for four minutes at least every three to six months (Weyant
        et al., 2013)

        The USPSTF [US Preventive Services Task Force] recommends that primary care clinicians
        apply fluoride varnish to the primary teeth of all infants and children starting at the age
        of primary tooth eruption. (B recommendation)
        """
        pass



from protocols import events
from protocols.protocol import ClinicalQualityMeasure
from protocols.value_set.v2021 import (
    EncounterInpatient, Ethnicity, Female, HomeHealthcareServices, HospiceCareAmbulatory, HpvTest,
    OfficeVisit, OncAdministrativeSex, PapTest, Payer,
    PreventiveCareServicesEstablishedOfficeVisit18AndUp,
    PreventiveCareServicesInitialOfficeVisit18AndUp, Race)


class ClinicalQualityMeasure124v9(ClinicalQualityMeasure):
    """
    Cervical Cancer Screening

    Description: Percentage of women 21-64 years of age who were screened for cervical cancer using
    either of the following criteria:
    *  Women age 21-64 who had cervical cytology performed within the last 3 years
    *  Women age 30-64 who had cervical human papillomavirus (HPV) testing performed within the
    last 5 years

    Definition: None

    Rationale: All women are at risk for cervical cancer. In 2019, an estimated 13,170 women were
    diagnosed with cervical cancer in the U.S., resulting in an estimated 4,250 deaths (National
    Cancer Institute, 2019). If pre-cancerous lesions are detected early by Pap tests and treated,
    the likelihood of survival is nearly 100 percent (American Cancer Society, 2019).

    Guidance: To ensure the measure is only looking for a cervical cytology test only after a woman
    turns 21 years of age, the youngest age in the initial population is 23.

    Patient self-report for procedures as well as diagnostic studies should be recorded in
    'Procedure, Performed' template or 'Diagnostic Study, Performed' template in QRDA-1.

    This eCQM is a patient-based measure.

    This version of the eCQM uses QDM version 5.5.  Please refer to the eCQI resource center
    (https://ecqi.healthit.gov/qdm) for more information on the QDM.

    More information: https://ecqi.healthit.gov/ecqm/measures/cms124v9
    """

    title = 'Cervical Cancer Screening'

    identifiers = ['CMS124v9']

    types = ['CQM']

    responds_to_event_types = [
        events.HEALTH_MAINTENANCE,
    ]

    authors = [
        'National Committee for Quality Assurance',
    ]

    references = [
        'American Cancer Society. (2019). Cancer Prevention & Early Detection Facts & Figures 2019-2020. Atlanta: American Cancer Society.',
        'National Cancer Institute. (2019). SEER Cancer Statistics Review, 1975-2016. Retrieved October 17, 2019, from SEER website: https://seer.cancer.gov/csr/1975_2016/index.html',
        'US Preventive Services Task Force, Curry, S. J., Krist, A. H., Owens, D. K., Barry, M. J., Caughey, A. B., … Wong, J. B. (2018). Screening for Cervical Cancer: US Preventive Services Task Force Recommendation Statement. JAMA, 320(7), 674–686. https://doi.org/10.1001/jama.2018.10897',
    ]

    funding_source = ''

    def in_initial_population(self):
        """
        Initial population: Women 23-64 years of age with a visit during the measurement period
        """
        pass

    def in_denominator(self):
        """
        Denominator: Equals Initial Population

        Exclusions: Women who had a hysterectomy with no residual cervix or a congenital absence of
        cervix.

        Exclude patients whose hospice care overlaps the measurement period.

        Exceptions: None
        """
        return self.in_initial_population()

    def in_numerator(self):
        """
        Numerator: Women with one or more screenings for cervical cancer. Appropriate screenings
        are defined by any one of the following criteria:
        *  Cervical cytology performed during the measurement period or the two years prior to the
        measurement period for women who are at least 21 years old at the time of the test

        *  Cervical human papillomavirus (HPV) testing performed during the measurement period or
        the four years prior to the measurement period for women who are 30 years or older at the
        time of the test

        Exclusions: Not Applicable
        """
        pass


    def compute_results(self):
        """
        Clinical recommendation: US Preventive Services Task Force (USPSTF) (2018) "The USPSTF
        recommends screening for cervical cancer every 3 years with cervical cytology alone in
        women aged 21 to 29 years. For women aged 30 to 65 years, the USPSTF recommends screening
        every 3 years with cervical cytology alone, every 5 years with high-risk human
        papillomavirus (hrHPV) testing alone, or every 5 years with hrHPV testing in combination
        with cytology (cotesting) (A recommendation)"

        "The USPSTF recommends against screening for cervical cancer in women older than 65 years
        who have had adequate prior screening and are not otherwise at high risk for cervical
        cancer. (D recommendation)"

        "The USPSTF recommends against screening for cervical cancer in women younger than 21
        years. (D recommendation)"

        "The USPSTF recommends against screening for cervical cancer in women who have had a
        hysterectomy with removal of the cervix and do not have a history of a high-grade
        precancerous lesion (ie, cervical intraepithelial neoplasia [CIN] grade 2 or 3) or cervical
        cancer. (D recommendation)"
        """
        pass
